// **************************************************************** 
//
//   IBM Confidential
//   OCO Source Materials
//   <<PART NUMBER - 5724-B75>>
//   (C) Copyright IBM Corp. 2001 - All Rights Reserved.
//   US Government Users Restricted Rights - Use, duplication or disclosure
//   restricted by GSA ADP Schedule Contract with IBM Corp.
//
// ****************************************************************


package com.ibm.etools.cobol.importer;

import java.io.*;
import java.util.*;
import com.ibm.record.*;
import java.math.*;
import java.net.MalformedURLException;


//#line 18 "CobolParser.java"




//#####################################################################
// class: CobolParser
// does : encapsulates yacc() parser functionality in a Java
//        class for quick code development
//#####################################################################
public class CobolParser
{

boolean yydebug;        //do I want debug output?
int yynerrs;            //number of errors so far
int yyerrflag;          //was there an error?
int yychar;             //the current working character
String yytext;          //string buffer for use by yylex()

//########## MESSAGES ##########
//###############################################################
// method: debug
//###############################################################
void debug(String msg)
{
  if (yydebug)
    System.out.println(msg);
}

//########## STATE STACK ##########
final static int YYSTACKSIZE = 500;  //maximum stack size
int statestk[],stateptr;             //state stack
//###############################################################
// methods: state stack push,pop,drop,peek
//###############################################################
void state_push(int state)
{
  if (stateptr>=YYSTACKSIZE)         //overflowed?
    return;
  statestk[++stateptr]=state;
}
int state_pop()
{
  if (stateptr<0)                    //underflowed?
    return -1;
  return statestk[stateptr--];
}
void state_drop(int cnt)
{
int ptr;
  ptr=stateptr-cnt;
  if (ptr<0)
    return;
  stateptr = ptr;
}
int state_peek(int relative)
{
int ptr;
  ptr=stateptr-relative;
  if (ptr<0)
    return -1;
  return statestk[ptr];
}
//###############################################################
// method: init_stacks : allocate and prepare stacks
//###############################################################
boolean init_stacks()
{
  statestk = new int[YYSTACKSIZE];
  stateptr = -1;
  val_init();
  return true;
}
//###############################################################
// method: dump_stacks : show n levels of the stacks
//###############################################################
void dump_stacks(int count)
{
int i;
  System.out.println("=index==state====value=     s:"+stateptr+"  v:"+valptr);
  for (i=0;i<count;i++)
    System.out.println(" "+i+"    "+statestk[i]+"      "+valstk[i]);
  System.out.println("======================");
}
//########## SEMANTIC VALUES ########## type:int
int yyval;           //used to return semantic vals from action routines
int yylval;          //the 'lval' (result) I got from yylex()
int valstk[];
int valptr;
//###############################################################
// methods: value stack push,pop,drop,peek.  semantic type=int
//###############################################################
void val_init()
{
  valstk=new int[YYSTACKSIZE];
  valptr=-1;
}
void val_push(int val)
{
  if (valptr>=YYSTACKSIZE)
    return;
  valstk[++valptr]=val;
}
int val_pop()
{
  if (valptr<0)
    return -1;
  return valstk[valptr--];
}
void val_drop(int cnt)
{
int ptr;
  ptr=valptr-cnt;
  if (ptr<0)
    return;
  valptr = ptr;
}
int val_peek(int relative)
{
int ptr;
  ptr=valptr-relative;
  if (ptr<0)
    return -1;
  return valstk[ptr];
}
//#### end semantic value section ####
public final static short FILLER=257;
public final static short COMP=258;
public final static short COMP1=259;
public final static short COMP2=260;
public final static short COMP3=261;
public final static short COMP5=262;
public final static short COMPX=263;
public final static short INDEX=264;
public final static short DISPLAY=265;
public final static short POINTER=266;
public final static short DISPLAY1=267;
public final static short NATIONAL=268;
public final static short PROCPOINTER=269;
public final static short OCCURS=270;
public final static short ASCENDING=271;
public final static short DESCENDING=272;
public final static short INDEXED=273;
public final static short DEPENDING=274;
public final static short REDEFINES=275;
public final static short TO=276;
public final static short LEADING=277;
public final static short TRAILING=278;
public final static short SEPARATE=279;
public final static short SYNC=280;
public final static short RIGHT=281;
public final static short LEFT=282;
public final static short JUST=283;
public final static short BLANK=284;
public final static short ZERO=285;
public final static short SPACE=286;
public final static short QUOTE=287;
public final static short SQUOTE=288;
public final static short HIGHVALUE=289;
public final static short LOWVALUE=290;
public final static short NULL=291;
public final static short PERIOD=292;
public final static short INTEGER=293;
public final static short ID=294;
public final static short PICTURE=295;
public final static short VALUE=296;
public final static short ALL=297;
public final static short THROUGH=298;
public final static short COPY=299;
public final static short OFIN=300;
public final static short REPLACING=301;
public final static short REPLACINGID=302;
public final static short OPTIONAL=303;
public final static short LITERAL=304;
public final static short YYERRCODE=256;
final static short yylhs[] = {                           -1,
    0,    1,    1,    4,    2,    6,    2,   10,    3,    9,
    9,    9,    5,    5,   11,   11,   11,   11,   11,   11,
   11,   11,   11,   20,   20,   15,   19,   17,   21,   21,
   21,   24,   24,   24,   25,   22,   22,   26,   26,   23,
   23,   28,   27,   27,   18,   30,   30,   30,   30,   30,
   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,
   30,   30,   30,   30,   29,   31,   29,   29,   12,   12,
   12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
   16,   16,   16,   16,   13,   13,   13,   14,   14,    7,
    7,    8,    8,   32,   32,
};
final static short yylen[] = {                            2,
    1,    2,    0,    0,    4,    0,    6,    0,    3,    1,
    1,    0,    2,    0,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    2,    2,    2,    2,    5,    4,    2,
    0,    3,    1,    0,    1,    2,    0,    2,    2,    2,
    0,    2,    2,    0,    2,    2,    3,    1,    2,    1,
    1,    1,    1,    1,    1,    1,    2,    2,    2,    2,
    2,    2,    2,    1,    2,    0,    5,    0,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    2,    1,    2,    1,    2,    2,    1,    2,    2,
    0,    2,    0,    3,    0,
};
final static short yydefred[] = {                         3,
    0,    0,    6,    2,    0,    0,    8,   14,    0,    0,
    0,    0,    0,   11,   10,    9,   69,   70,   71,   72,
   73,   75,   76,   74,   77,   78,   79,   80,    0,    0,
    0,    0,    0,    0,    0,    5,    0,   68,   13,   15,
   16,   17,   18,   19,   20,   21,   22,   23,   90,   95,
    0,    0,   27,   82,   84,   87,   86,   89,   26,   25,
   24,    0,    0,    7,    0,    0,   37,   50,   51,   52,
   53,   54,   55,   56,   64,    0,    0,   48,    0,    0,
   35,    0,   33,    0,    0,   46,   57,   58,   59,   60,
   61,   62,   63,    0,   49,    0,   94,    0,    0,   44,
   44,    0,   36,   47,    0,   32,    0,    0,    0,   44,
   40,   67,   43,    0,
};
final static short yydgoto[] = {                          1,
    2,    4,    8,    5,   11,    6,   13,   51,   16,   10,
   39,   40,   41,   42,   43,   44,   45,   46,   47,   48,
   67,   85,  102,   82,   83,  103,  108,  111,   62,   79,
   96,   63,
};
final static short yysindex[] = {                         0,
    0, -299,    0,    0, -278, -274,    0,    0, -282, -214,
  115, -281, -254,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, -248, -229,
 -212, -198, -205, -211, -203,    0, -221,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 -208, -230,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  400, -217,    0, -206, -204,    0,    0,    0,    0,
    0,    0,    0,    0,    0, -196,  420,    0,    0, -192,
    0, -210,    0, -161, -193,    0,    0,    0,    0,    0,
    0,    0,    0, -189,    0, -182,    0, -206, -206,    0,
    0, -154,    0,    0,  400,    0, -210, -174, -174,    0,
    0,    0,    0, -174,
};
final static short yyrindex[] = {                         0,
    0,    1,    0,    0,    0,    0,    0,    0, -276,  154,
    0,    0, -171,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  193,  232,  271,  310,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, -166,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  349, -170,    0, -209,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, -256,    0,
    0, -127,    0,    0,   37,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, -209,    0,
    0,  388,    0,    0,    0,    0,  -88,  -49,  -10,    0,
    0,    0,    0,   76,
};
final static short yygindex[] = {                         0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   24,   26,    0,  -84,    0,    0,   20,
    0,    0,
};
final static int YYTABLESIZE=724;
final static short yytable[] = {                          3,
    1,   65,   65,   65,   65,   65,   65,   65,   65,   65,
   65,   65,   65,   65,    7,   91,  109,   12,   65,    9,
   65,   65,   49,   65,   91,  114,   65,   65,   65,   65,
   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,
   65,   66,   14,   65,   52,   66,   50,   65,   34,   34,
   34,   34,   34,   34,   34,   34,   34,   34,   34,   34,
   34,   34,   34,   34,   53,   34,   54,   34,   34,   58,
   34,   60,   61,   34,   34,   56,   57,  100,  101,   15,
   55,   59,   34,   64,   80,   34,   34,   81,   84,   98,
   34,   31,   31,   31,   31,   31,   31,   31,   31,   31,
   31,   31,   31,   31,   31,   31,   31,   86,   31,   97,
   31,   31,   99,   31,  104,  105,   31,   31,  110,  113,
   93,   92,  107,  106,  112,   31,    0,    0,   31,   31,
   30,   30,   30,   30,   30,   30,   30,   30,   30,   30,
   30,   30,   30,   30,   30,   30,    0,   30,    0,   30,
   30,    0,   30,    0,    0,   30,   30,    0,    0,    0,
    0,    0,    0,    0,   30,    0,    0,   30,   30,   29,
   29,   29,   29,   29,   29,   29,   29,   29,   29,   29,
   29,   29,   29,   29,   29,    0,   29,    0,   29,   29,
    0,   29,    0,    0,   29,   29,    0,    0,    0,    0,
    0,    0,    0,   29,    0,    0,   29,   29,   38,   38,
   38,   38,   38,   38,   38,   38,   38,   38,   38,   38,
   38,   38,   38,   38,    0,   38,    0,   38,   38,    0,
   38,    0,    0,   38,   38,    0,    0,    0,    0,    0,
    0,    0,   38,    0,    0,   38,   38,   39,   39,   39,
   39,   39,   39,   39,   39,   39,   39,   39,   39,   39,
   39,   39,   39,    0,   39,    0,   39,   39,    0,   39,
    0,    0,   39,   39,    0,    0,    0,    0,    0,    0,
    0,   39,    0,    0,   39,   39,    0,    0,    0,    0,
    0,    0,    0,    4,   41,   41,   41,   41,   41,   41,
   41,   41,   41,   41,   41,   41,   41,    0,    0,   41,
    0,   41,    0,   41,   41,    0,   41,    0,    0,   41,
   41,    0,    0,    0,    0,    0,    0,    0,   41,    0,
    0,   41,   41,   42,   42,   42,   42,   42,   42,   42,
   42,   42,   42,   42,   42,   42,    0,    0,   42,    0,
   42,    0,   42,   42,    0,   42,    0,    0,   42,   42,
    0,    0,    0,    0,    0,    0,    0,   42,    0,    0,
   42,   42,   17,   18,   19,   20,   21,   22,   23,   24,
   25,   26,   27,   28,   29,    0,    0,    0,    0,   30,
    0,   31,   32,    0,   33,    0,    0,   34,   35,    0,
    0,    0,    0,    0,    0,    0,   36,    0,    0,   37,
   38,   12,   12,   12,   12,   12,   12,   12,   12,   12,
   12,   12,   12,   12,    0,    0,    0,    0,   12,    0,
   12,   12,    0,   12,    0,    0,   12,   12,    0,    0,
    0,    0,    0,    0,    0,   12,    0,    0,   12,   12,
   81,   81,   81,   81,   81,   81,   81,   81,   81,   81,
   81,   81,   81,    0,    0,    0,    0,   81,    0,   81,
   81,    0,   81,    0,    0,   81,   81,    0,    0,    0,
    0,    0,    0,    0,   81,    0,    0,   81,   81,   83,
   83,   83,   83,   83,   83,   83,   83,   83,   83,   83,
   83,   83,    0,    0,    0,    0,   83,    0,   83,   83,
    0,   83,    0,    0,   83,   83,    0,    0,    0,    0,
    0,    0,    0,   83,    0,    0,   83,   83,   85,   85,
   85,   85,   85,   85,   85,   85,   85,   85,   85,   85,
   85,    0,    0,    0,    0,   85,    0,   85,   85,    0,
   85,    0,    0,   85,   85,    0,    0,    0,    0,    0,
    0,    0,   85,    0,    0,   85,   85,   88,   88,   88,
   88,   88,   88,   88,   88,   88,   88,   88,   88,   88,
    0,    0,    0,    0,   88,    0,   88,   88,    0,   88,
    0,    0,   88,   88,    0,    0,    0,    0,    0,    0,
    0,   88,    0,    0,   88,   88,   45,   45,   45,   45,
   45,   45,   45,   45,   45,   45,   45,   45,   45,    0,
    0,    0,    0,   45,    0,   45,   45,    0,   45,    0,
    0,   45,   45,    0,    0,    0,    0,    0,    0,    0,
   45,    0,    0,   45,   45,   28,   28,   28,   28,   28,
   28,   28,   28,   28,   28,   28,   28,   28,    0,    0,
    0,    0,   28,    0,   28,   28,    0,   28,    0,    0,
   28,   28,    0,    0,    0,    0,    0,    0,    0,   28,
    0,    0,   28,   28,   68,   69,   70,   71,   72,   73,
   74,    0,   75,   76,    0,    0,   77,    0,    0,    0,
    0,    0,    0,   78,   87,   88,   89,   90,   91,   92,
   93,    0,    0,   94,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   95,
};
final static short yycheck[] = {                        299,
    0,  258,  259,  260,  261,  262,  263,  264,  265,  266,
  267,  268,  269,  270,  293,  292,  101,  300,  275,  294,
  277,  278,  304,  280,  301,  110,  283,  284,  285,  286,
  287,  288,  289,  290,  291,  292,  293,  294,  295,  296,
  297,  298,  257,  274,  293,  276,  301,  304,  258,  259,
  260,  261,  262,  263,  264,  265,  266,  267,  268,  269,
  270,  271,  272,  273,  294,  275,  279,  277,  278,  281,
  280,  293,  294,  283,  284,  281,  282,  271,  272,  294,
  279,  285,  292,  292,  302,  295,  296,  294,  293,  300,
  300,  258,  259,  260,  261,  262,  263,  264,  265,  266,
  267,  268,  269,  270,  271,  272,  273,  304,  275,  302,
  277,  278,  274,  280,  304,  298,  283,  284,  273,  294,
  292,  292,   99,   98,  105,  292,   -1,   -1,  295,  296,
  258,  259,  260,  261,  262,  263,  264,  265,  266,  267,
  268,  269,  270,  271,  272,  273,   -1,  275,   -1,  277,
  278,   -1,  280,   -1,   -1,  283,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  292,   -1,   -1,  295,  296,  258,
  259,  260,  261,  262,  263,  264,  265,  266,  267,  268,
  269,  270,  271,  272,  273,   -1,  275,   -1,  277,  278,
   -1,  280,   -1,   -1,  283,  284,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  292,   -1,   -1,  295,  296,  258,  259,
  260,  261,  262,  263,  264,  265,  266,  267,  268,  269,
  270,  271,  272,  273,   -1,  275,   -1,  277,  278,   -1,
  280,   -1,   -1,  283,  284,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  292,   -1,   -1,  295,  296,  258,  259,  260,
  261,  262,  263,  264,  265,  266,  267,  268,  269,  270,
  271,  272,  273,   -1,  275,   -1,  277,  278,   -1,  280,
   -1,   -1,  283,  284,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  292,   -1,   -1,  295,  296,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  293,  258,  259,  260,  261,  262,  263,
  264,  265,  266,  267,  268,  269,  270,   -1,   -1,  273,
   -1,  275,   -1,  277,  278,   -1,  280,   -1,   -1,  283,
  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  292,   -1,
   -1,  295,  296,  258,  259,  260,  261,  262,  263,  264,
  265,  266,  267,  268,  269,  270,   -1,   -1,  273,   -1,
  275,   -1,  277,  278,   -1,  280,   -1,   -1,  283,  284,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  292,   -1,   -1,
  295,  296,  258,  259,  260,  261,  262,  263,  264,  265,
  266,  267,  268,  269,  270,   -1,   -1,   -1,   -1,  275,
   -1,  277,  278,   -1,  280,   -1,   -1,  283,  284,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  292,   -1,   -1,  295,
  296,  258,  259,  260,  261,  262,  263,  264,  265,  266,
  267,  268,  269,  270,   -1,   -1,   -1,   -1,  275,   -1,
  277,  278,   -1,  280,   -1,   -1,  283,  284,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  292,   -1,   -1,  295,  296,
  258,  259,  260,  261,  262,  263,  264,  265,  266,  267,
  268,  269,  270,   -1,   -1,   -1,   -1,  275,   -1,  277,
  278,   -1,  280,   -1,   -1,  283,  284,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  292,   -1,   -1,  295,  296,  258,
  259,  260,  261,  262,  263,  264,  265,  266,  267,  268,
  269,  270,   -1,   -1,   -1,   -1,  275,   -1,  277,  278,
   -1,  280,   -1,   -1,  283,  284,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  292,   -1,   -1,  295,  296,  258,  259,
  260,  261,  262,  263,  264,  265,  266,  267,  268,  269,
  270,   -1,   -1,   -1,   -1,  275,   -1,  277,  278,   -1,
  280,   -1,   -1,  283,  284,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  292,   -1,   -1,  295,  296,  258,  259,  260,
  261,  262,  263,  264,  265,  266,  267,  268,  269,  270,
   -1,   -1,   -1,   -1,  275,   -1,  277,  278,   -1,  280,
   -1,   -1,  283,  284,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  292,   -1,   -1,  295,  296,  258,  259,  260,  261,
  262,  263,  264,  265,  266,  267,  268,  269,  270,   -1,
   -1,   -1,   -1,  275,   -1,  277,  278,   -1,  280,   -1,
   -1,  283,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  292,   -1,   -1,  295,  296,  258,  259,  260,  261,  262,
  263,  264,  265,  266,  267,  268,  269,  270,   -1,   -1,
   -1,   -1,  275,   -1,  277,  278,   -1,  280,   -1,   -1,
  283,  284,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  292,
   -1,   -1,  295,  296,  285,  286,  287,  288,  289,  290,
  291,   -1,  293,  294,   -1,   -1,  297,   -1,   -1,   -1,
   -1,   -1,   -1,  304,  285,  286,  287,  288,  289,  290,
  291,   -1,   -1,  294,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  304,
};
final static short YYFINAL=1;
final static short YYMAXTOKEN=304;
final static String yyname[] = {
"end-of-file",null,null,null,null,null,null,null,null,null,null,null,null,null,
null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,
null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,
null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,
null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,
null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,
null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,
null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,
null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,
null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,
null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,
null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,
null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,
null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,
null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,
null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,
null,null,null,"FILLER","COMP","COMP1","COMP2","COMP3","COMP5","COMPX","INDEX",
"DISPLAY","POINTER","DISPLAY1","NATIONAL","PROCPOINTER","OCCURS","ASCENDING",
"DESCENDING","INDEXED","DEPENDING","REDEFINES","TO","LEADING","TRAILING",
"SEPARATE","SYNC","RIGHT","LEFT","JUST","BLANK","ZERO","SPACE","QUOTE","SQUOTE",
"HIGHVALUE","LOWVALUE","NULL","PERIOD","INTEGER","ID","PICTURE","VALUE","ALL",
"THROUGH","COPY","OFIN","REPLACING","REPLACINGID","OPTIONAL","LITERAL",
};
final static String yyrule[] = {
"$accept : commarea",
"commarea : records",
"records : records record",
"records :",
"$$1 :",
"record : $$1 descriptor clauses PERIOD",
"$$2 :",
"record : COPY $$2 ID ofinClause replacingClause PERIOD",
"$$3 :",
"descriptor : INTEGER $$3 name",
"name : ID",
"name : FILLER",
"name :",
"clauses : clauses clause",
"clauses :",
"clause : usageClause",
"clause : synchronizedClause",
"clause : justificationClause",
"clause : blankClause",
"clause : signClause",
"clause : occursClause",
"clause : valueClause",
"clause : redefinesClause",
"clause : pictureClause",
"pictureClause : PICTURE ID",
"pictureClause : PICTURE INTEGER",
"blankClause : BLANK ZERO",
"redefinesClause : REDEFINES ID",
"occursClause : OCCURS INTEGER dependingClause keyClauses indexClauses",
"dependingClause : TO INTEGER DEPENDING qualifiedIDs",
"dependingClause : DEPENDING qualifiedIDs",
"dependingClause :",
"qualifiedIDs : qualifiedIDs OFIN qualifiedID",
"qualifiedIDs : qualifiedID",
"qualifiedIDs :",
"qualifiedID : ID",
"keyClauses : keyClauses keyClause",
"keyClauses :",
"keyClause : ASCENDING idList",
"keyClause : DESCENDING idList",
"indexClauses : indexClauses indexClause",
"indexClauses :",
"indexClause : INDEXED idList",
"idList : idList ID",
"idList :",
"valueClause : VALUE valueList",
"literal : ID LITERAL",
"literal : ALL ID LITERAL",
"literal : LITERAL",
"literal : ALL LITERAL",
"literal : ZERO",
"literal : SPACE",
"literal : QUOTE",
"literal : SQUOTE",
"literal : HIGHVALUE",
"literal : LOWVALUE",
"literal : NULL",
"literal : ALL ZERO",
"literal : ALL SPACE",
"literal : ALL QUOTE",
"literal : ALL SQUOTE",
"literal : ALL HIGHVALUE",
"literal : ALL LOWVALUE",
"literal : ALL NULL",
"literal : INTEGER",
"valueList : valueList literal",
"$$4 :",
"valueList : valueList literal $$4 THROUGH literal",
"valueList :",
"usageClause : COMP",
"usageClause : COMP1",
"usageClause : COMP2",
"usageClause : COMP3",
"usageClause : COMP5",
"usageClause : DISPLAY",
"usageClause : COMPX",
"usageClause : INDEX",
"usageClause : POINTER",
"usageClause : DISPLAY1",
"usageClause : NATIONAL",
"usageClause : PROCPOINTER",
"signClause : LEADING",
"signClause : LEADING SEPARATE",
"signClause : TRAILING",
"signClause : TRAILING SEPARATE",
"synchronizedClause : SYNC",
"synchronizedClause : SYNC LEFT",
"synchronizedClause : SYNC RIGHT",
"justificationClause : JUST",
"justificationClause : JUST RIGHT",
"ofinClause : OFIN LITERAL",
"ofinClause :",
"replacingClause : REPLACING replacingList",
"replacingClause :",
"replacingList : replacingList REPLACINGID REPLACINGID",
"replacingList :",
};

//#line 715 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"


   void initInfo()
   {
      usage = CobolRecord.NOUSAGE;
      sign = CobolRecord.NOSIGN;

      align = false;
      fill = false;
      // 1.0.1 NL changes
      inDBCS = false;
      // 1.0.1 NL end
      isSign = false;
      decimal = false;
      numeric = true;
      paddingRequired = false;

      bound = 1;
      lowBound = -1; /*26260 GIM Save lower bound for ODO */
      isLowBound = -1; /*26260 GIM Save lower bound for ODO */
      numberOfNines = 0;
      decimalPosition = 0;
      scale = 0;
      picFieldLength = 0;

      value = null;
      figConst = null;
      redefinedItem = null;
      picString = new String();
      javaType = new String();

      idVector = new Vector();
      possibleValues = new Vector();
   }

   void storeInfo() throws CobolException
   {
      if (level == 88)
      {
         ((CobolRecord)groupStack.peek()).setPossibleValues(name, possibleValues);
      }
      else
      {
         previousType=javaType;
         while (groupStack.peek() instanceof CobolElementaryRecord)
            groupStack.pop();

         // determine which group record belongs to
         while ( level <= ((CobolRecord)groupStack.peek()).level() )
            groupStack.pop();

         if ( !picString.equals("") )
         {
            countForIMS++;
            pNewRecord =  new CobolElementaryRecord((CobolRecord)groupStack.peek(), name, level);
            if (groupStack.peek() instanceof CobolGroupRecord)
               ((CobolGroupRecord)groupStack.peek()).add(pNewRecord);
            else if (groupStack.peek() instanceof CobolElementaryRecord)
               ((CobolElementaryRecord)groupStack.peek()).add(pNewRecord);
            pNewRecord.setPicString(picString);
 //           pNewRecord.setNewPicClass(com.ibm.ivj.eab.tools.ClassPathUtility.loadClass(CobolImport.workspace, javaType));
            if ( value != null )
               pNewRecord.setValue(value);

            if ( figConst != null )
               pNewRecord.setFigConst(figConst);


            // Add the TranCode Field as the 3rd field for an IMS record
	    if (countForIMS == 2 && !IMSTranCode.equals(""))
            {
		StringTokenizer st = new StringTokenizer(IMSTranCode, ":");
		st.nextToken();
		CobolElementaryRecord tranCodeRec = new CobolElementaryRecord((CobolRecord)groupStack.peek(), st.nextToken(),level);
		if (groupStack.peek() instanceof CobolGroupRecord)
		    ((CobolGroupRecord)groupStack.peek()).add(tranCodeRec);
		else if (groupStack.peek() instanceof CobolElementaryRecord)
			((CobolElementaryRecord)groupStack.peek()).add(tranCodeRec);
	   	if (st.hasMoreTokens())
		   tranCodeRec.setPicString("X("+st.nextToken()+")");
	   	else tranCodeRec.setPicString("X(8)");
//tranCodeRec.setNewPicClass(com.ibm.ivj.eab.tools.ClassPathUtility.loadClass(CobolImport.workspace, javaType));
		if ( st.hasMoreTokens())
	   	   tranCodeRec.setValue(st.nextToken());
				
		groupStack.push(pNewRecord);
			   	
		tranCodeRec.setRedefinedItem(null);
		tranCodeRec.setDependedOnItem(new Vector());
		tranCodeRec.setUsage(CobolRecord.NOUSAGE);
		tranCodeRec.setSign(CobolRecord.NOSIGN);
		tranCodeRec.setAlignment(false);
		tranCodeRec.setFiller(false);
		tranCodeRec.setArrayBound(1);
		groupStack.push(tranCodeRec);
	    }
            else groupStack.push(pNewRecord);
         }
         else
         {
            pNewRecord = new CobolGroupRecord((CobolRecord)groupStack.peek(), name, level);
            if (groupStack.peek() instanceof CobolGroupRecord)
               ((CobolGroupRecord)groupStack.peek()).add(pNewRecord);
            else if (groupStack.peek() instanceof CobolElementaryRecord)
               ((CobolElementaryRecord)groupStack.peek()).add(pNewRecord);
            // create new level
            groupStack.push(pNewRecord);
         }
         pNewRecord.setRedefinedItem(redefinedItem);
         pNewRecord.setDependedOnItem(idVector);
         pNewRecord.setUsage(usage);
         pNewRecord.setSign(sign);
         pNewRecord.setAlignment(align);
         pNewRecord.setFiller(fill);
         pNewRecord.setArrayBound(bound);
         pNewRecord.setArrayLowBound(lowBound); /*26260 GIM Save lower bound for ODO */
      }
   }

public static String hyphenToUnderscore(String str)
{
   // change all hyphens to underscores since COBOL allows hyphens in identifiers
   // ensure name does not begin with a numeric as COBOL allows
   StringTokenizer st = new StringTokenizer(str, "-0123456789", true);
   boolean checkNumber = true;
   StringBuffer strBuff = new StringBuffer();
   while ( st.hasMoreTokens() )
   {
      String token = st.nextToken();
      if ( token.equals("-") )
         strBuff.append("__");
      else
      {
         if ( checkNumber )
         {
            try {
               Integer.parseInt(token);
               strBuff.append("__");
               strBuff.append(token);
            } catch (NumberFormatException e) {
               strBuff.append(token);
            }
         }
         else
            strBuff.append(token);
      }
      checkNumber = false;
   }
   return strBuff.toString();
}

public int yylex() throws CobolException
{
	StreamTokenizer f = (StreamTokenizer)tokenizerList.peek();
	f.ordinaryChars(32, 126);
	f.wordChars(32, 126);
	f.whitespaceChars(' ',' ');
	if ( token != OFIN || (token == OFIN && !bCopyContext))      /*16494 Keep old semantics for OF/IN in COPY statement*/
	{
		f.quoteChar('\'');
		f.quoteChar('"');
	}
	
	int ttype;
	try {
		ttype = f.nextToken();
	} catch (IOException exc) {
		throw new CobolException(CobolResource.getString("_EXC_Import_failed"));
	}
		switch (ttype)
		{
			case java.io.StreamTokenizer.TT_EOF :
				if (tokenizerList.size() > 1)
				{
					tokenizerList.pop();
					cobolFileList.pop();
					return (yylex());
				}
				return 0;
			case java.io.StreamTokenizer.TT_WORD:
				int rc = findSymbol(f.sval);
				if (rc == -1)
				{
					if (f.sval.endsWith(",") || f.sval.endsWith(";"))
					{
					f.sval = f.sval.substring(0, f.sval.length()-1);
					rc = findSymbol(f.sval);
					}
				}

				if ( rc == OPTIONAL )
				{
					if ( f.sval.equalsIgnoreCase("BY") && token == REPLACING )
					{
						token = REPLACINGID;
						/***
						if ( CobolImport.DEBUG )
						{
							String str = operand1.firstElement().toString();
							for (int k=1;k<operand1.size();k++)
								str = str + " " + operand1.elementAt(k);
							System.out.println("REPLACINGID: "+str);
						}
						 ***/
						yylval = buffer.put(operand1);
			   			return REPLACINGID;
					}
					else
						return (yylex());
				}
				else if ( rc == PERIOD )
				{
					if ( token == REPLACINGID )
					{
						f.pushBack();
						token = -1;
							/***
							if ( CobolImport.DEBUG )
							{
								String str = operand2.firstElement().toString();
								for (int k=1;k<operand2.size();k++)
									str = str + " " + operand2.elementAt(k);
								System.out.println("REPLACINGBYID: "+str);
							}
							 ***/
						yylval = buffer.put(operand2);
		   				return REPLACINGID;
					}
					else
					{
						/***
						if ( CobolImport.DEBUG )
							System.out.println("PERIOD");
						 ***/
						token = rc;
						return rc;
					}
				}
				else if ( token == REPLACING )
				{
					operand1.addElement(f.sval);
					return (yylex());
				}
				else if ( token == REPLACINGID )
				{
					if ( operand2.isEmpty() )
					{
						operand2.addElement(f.sval);
						if ( !f.sval.startsWith("==") )
						{
							token = REPLACING;
							/***
							if ( CobolImport.DEBUG )
							{
								String str = operand2.firstElement().toString();
								for (int k=1;k<operand2.size();k++)
									str = str + " " + operand2.elementAt(k);
								System.out.println("REPLACINGBYID1: "+str);
							}
							 ***/
							yylval = buffer.put(operand2);
	   						return REPLACINGID;
						}
						else
							return (yylex());
					}
					else if ( operand2.lastElement().toString().endsWith("==") && operand2.size() > 1 )
					{
						f.pushBack();
						token = REPLACING;
						/***
						if ( CobolImport.DEBUG )
						{
							String str = operand2.firstElement().toString();
							for (int k=1;k<operand2.size();k++)
								str = str + " " + operand2.elementAt(k);
							System.out.println("REPLACINGBYID2: "+str);
						}
						 ***/
						yylval = buffer.put(operand2);
   						return REPLACINGID;
					}
					else if ( operand2.lastElement().toString().endsWith("==") && operand2.firstElement().toString().length() > 2 )
					{
						f.pushBack();
						token = REPLACING;
						/***
						if ( CobolImport.DEBUG )
						{
							String str = operand2.firstElement().toString();
							for (int k=1;k<operand2.size();k++)
								str = str + " " + operand2.elementAt(k);
							System.out.println("REPLACINGBYID3: "+str);
						}
						 ***/
						yylval = buffer.put(operand2);
   						return REPLACINGID;
					}
					else
					{
						operand2.addElement(f.sval);
						return (yylex());
					}
				}
				else if ( token == OFIN && bCopyContext )   /*16494 Keep old semantics for OF/IN in COPY statement*/
				{
					token = -1;
					/***
					if ( CobolImport.DEBUG )
						System.out.println("LITERAL: "+f.sval.substring(1, f.sval.length()-1));
					 ***/
					yylval = buffer.put(f.sval.substring(1, f.sval.length()-1));
					return LITERAL;
				}
				else if ( rc != -1 )
				{
					/***
					if ( CobolImport.DEBUG )
						System.out.println(f.sval.toUpperCase(java.util.Locale.US));
					 ***/
					token = rc;
					return rc;
				}
		
				else
				{
					
					try {
						yylval = Integer.parseInt(f.sval);
						/***
						if ( CobolImport.DEBUG )
							System.out.println("INTEGER: "+f.sval);
						 ***/
						return INTEGER;
					} catch (NumberFormatException exc) {
						try{
							Double doub = Double.valueOf(f.sval);
							yylval = buffer.put(f.sval);
							if (token != PICTURE)
							{
								/***
								if ( CobolImport.DEBUG )
									System.out.println("LITERAL1: "+f.sval);
								 ***/
								return LITERAL;
							}
							else
							{
								/***
								if ( CobolImport.DEBUG )
									System.out.println("ID: "+f.sval);
								 ***/
								return ID;
							}
						}
						catch (NumberFormatException ex) {
							yylval = buffer.put(f.sval);
							/***
							if ( CobolImport.DEBUG )
								System.out.println("ID1: "+f.sval);
							 ***/
							return ID;
						}
					}
				}
			default:
				if ( ttype == '\'' || ttype == '"' )
				{
					/***
					if ( CobolImport.DEBUG )
						System.out.println("LITERAL2: "+f.sval);
					 ***/
					yylval = buffer.put(f.sval);
					return LITERAL;
				}
				else
				{
					/***
					if ( CobolImport.DEBUG )
						System.out.println("ID2: "+new Character((char)ttype).toString());
					 ***/
					yylval = buffer.put(new Character((char)ttype).toString());
					return ID;
				}
		}
}

   void yyerror(String s, boolean isError)
   {
      CobolResource.display(s);
      if ( isError ) {
         errorsOccurred = true;
         errorList.add(s);
      }
   }

   boolean getFlag()
   {
      return errorsOccurred;
   }

   public void addSymbols()
   {
      symbols.addElement("filler");            symbols.addElement(new Integer(FILLER));
      symbols.addElement("binary");            symbols.addElement(new Integer(COMP));
      symbols.addElement("computational");     symbols.addElement(new Integer(COMP));
      symbols.addElement("comp");              symbols.addElement(new Integer(COMP));
      symbols.addElement("computational-1");   symbols.addElement(new Integer(COMP1));
      symbols.addElement("comp-1");            symbols.addElement(new Integer(COMP1));
      symbols.addElement("computational-2");   symbols.addElement(new Integer(COMP2));
      symbols.addElement("comp-2");            symbols.addElement(new Integer(COMP2));
      symbols.addElement("computational-3");   symbols.addElement(new Integer(COMP3));
      symbols.addElement("comp-3");            symbols.addElement(new Integer(COMP3));
      symbols.addElement("computational-4");   symbols.addElement(new Integer(COMP));
      symbols.addElement("comp-4");            symbols.addElement(new Integer(COMP));
      symbols.addElement("computational-5");   symbols.addElement(new Integer(COMP5));
      symbols.addElement("comp-5");            symbols.addElement(new Integer(COMP5));
      symbols.addElement("computational-x");   symbols.addElement(new Integer(COMPX));
      symbols.addElement("comp-x");            symbols.addElement(new Integer(COMPX));
      symbols.addElement("index");             symbols.addElement(new Integer(INDEX));
      symbols.addElement("packed-decimal");    symbols.addElement(new Integer(COMP3));
      symbols.addElement("display");           symbols.addElement(new Integer(DISPLAY));
      symbols.addElement("pointer");           symbols.addElement(new Integer(POINTER));
      symbols.addElement("display-1");         symbols.addElement(new Integer(DISPLAY1));
      symbols.addElement("national");          symbols.addElement(new Integer(NATIONAL));
      symbols.addElement("procedure-pointer"); symbols.addElement(new Integer(PROCPOINTER));
      symbols.addElement("occurs");            symbols.addElement(new Integer(OCCURS));
      symbols.addElement("depending");         symbols.addElement(new Integer(DEPENDING));
      symbols.addElement("to");                symbols.addElement(new Integer(TO));
      symbols.addElement("indexed");           symbols.addElement(new Integer(INDEXED));
      symbols.addElement("ascending");         symbols.addElement(new Integer(ASCENDING));
      symbols.addElement("descending");        symbols.addElement(new Integer(DESCENDING));
      symbols.addElement("trailing");          symbols.addElement(new Integer(TRAILING));
      symbols.addElement("leading");           symbols.addElement(new Integer(LEADING));
      symbols.addElement("separate");          symbols.addElement(new Integer(SEPARATE));
      symbols.addElement("synchronized");      symbols.addElement(new Integer(SYNC));
      symbols.addElement("sync");              symbols.addElement(new Integer(SYNC));
      symbols.addElement("left");              symbols.addElement(new Integer(LEFT));
      symbols.addElement("right");             symbols.addElement(new Integer(RIGHT));
      symbols.addElement("just");              symbols.addElement(new Integer(JUST));
      symbols.addElement("justified");         symbols.addElement(new Integer(JUST));
      symbols.addElement("blank");             symbols.addElement(new Integer(BLANK));
      symbols.addElement("zero");              symbols.addElement(new Integer(ZERO));
      symbols.addElement("zeros");             symbols.addElement(new Integer(ZERO));
      symbols.addElement("zeroes");            symbols.addElement(new Integer(ZERO));
      symbols.addElement("space");             symbols.addElement(new Integer(SPACE));
      symbols.addElement("spaces");            symbols.addElement(new Integer(SPACE));
      symbols.addElement("quote");             symbols.addElement(new Integer(QUOTE));
      symbols.addElement("quotes");            symbols.addElement(new Integer(QUOTE));
      symbols.addElement("squote");            symbols.addElement(new Integer(SQUOTE));
      symbols.addElement("squotes");           symbols.addElement(new Integer(SQUOTE));
      symbols.addElement("high-value");        symbols.addElement(new Integer(HIGHVALUE));
      symbols.addElement("high-values");       symbols.addElement(new Integer(HIGHVALUE));
      symbols.addElement("low-value");         symbols.addElement(new Integer(LOWVALUE));
      symbols.addElement("low-values");        symbols.addElement(new Integer(LOWVALUE));
      symbols.addElement("null");              symbols.addElement(new Integer(NULL));
      symbols.addElement("nulls");             symbols.addElement(new Integer(NULL));
      symbols.addElement(".");                 symbols.addElement(new Integer(PERIOD));
      symbols.addElement("redefines");         symbols.addElement(new Integer(REDEFINES));
      symbols.addElement("through");           symbols.addElement(new Integer(THROUGH));
      symbols.addElement("thru");              symbols.addElement(new Integer(THROUGH));
      symbols.addElement("copy");              symbols.addElement(new Integer(COPY));
      symbols.addElement("of");                symbols.addElement(new Integer(OFIN));
      symbols.addElement("in");                symbols.addElement(new Integer(OFIN));
      symbols.addElement("replacing");         symbols.addElement(new Integer(REPLACING));
      symbols.addElement("picture");           symbols.addElement(new Integer(PICTURE));
      symbols.addElement("pic");               symbols.addElement(new Integer(PICTURE));
      symbols.addElement("value");             symbols.addElement(new Integer(VALUE));
      symbols.addElement("values");            symbols.addElement(new Integer(VALUE));
      symbols.addElement("all");               symbols.addElement(new Integer(ALL));
      symbols.addElement("character");         symbols.addElement(new Integer(OPTIONAL));
      symbols.addElement("is");                symbols.addElement(new Integer(OPTIONAL));
      symbols.addElement("usage");             symbols.addElement(new Integer(OPTIONAL));
      symbols.addElement("times");             symbols.addElement(new Integer(OPTIONAL));
      symbols.addElement("key");               symbols.addElement(new Integer(OPTIONAL));
      symbols.addElement("by");                symbols.addElement(new Integer(OPTIONAL));
      symbols.addElement("on");                symbols.addElement(new Integer(OPTIONAL));
      symbols.addElement("sign");              symbols.addElement(new Integer(OPTIONAL));
      symbols.addElement("when");              symbols.addElement(new Integer(OPTIONAL));
      symbols.addElement("are");               symbols.addElement(new Integer(OPTIONAL));
      symbols.addElement("suppress");          symbols.addElement(new Integer(OPTIONAL));
      symbols.addElement(",");                 symbols.addElement(new Integer(OPTIONAL));
      symbols.addElement(";");                 symbols.addElement(new Integer(OPTIONAL));
   }

   public int findSymbol(String toFind)
   {
      for (int i=0;i<symbols.size();i+=2)
      {
         if (toFind.toLowerCase(java.util.Locale.US).compareTo((String)symbols.elementAt(i)) == 0)
            return ((Integer)(symbols.elementAt(i+1))).intValue();
      }
      return -1;
   }

   public void setIMSTranCode(String str) {
	IMSTranCode=str;
	}

   public CobolParser(CobolGroupRecord group, StreamTokenizer tokenizer, String cobolFile)
   {
	  groupStack.push(group);
	  tokenizerList.push(tokenizer);
	  cobolFileList.push(cobolFile);
	  addSymbols();
   }

/**
 * Initializes the picture table representing the
 * table on page 115 of GC26-4047-6. The
 * representation uses 0x01 if the pairing is
 * valid before the decimal and 0x10 if it is
 * valid after the decimal. The indeces to the
 * table are the integer values of the characters.
 */

 private static void cobolPictureTableInit()
 {

   cobolPictureTable['B']['B'] = 0x11;
   cobolPictureTable['B'][','] = 0x11;
   cobolPictureTable['B']['0'] = 0x11;
   cobolPictureTable['B']['/'] = 0x11;
   cobolPictureTable['B']['.'] = 0x11;
   cobolPictureTable['B']['C'] = 0x11;
   cobolPictureTable['B']['D'] = 0x11;
   cobolPictureTable['B']['E'] = 0x00;
   cobolPictureTable['B']['Z'] = 0x11;
   cobolPictureTable['B']['*'] = 0x11;
   cobolPictureTable['B']['+'] = 0x11;
   cobolPictureTable['B']['-'] = 0x11;
   cobolPictureTable['B']['$'] = 0x11;
   cobolPictureTable['B']['9'] = 0x11;
   cobolPictureTable['B']['A'] = 0x11;
   cobolPictureTable['B']['X'] = 0x11;
   cobolPictureTable['B']['S'] = 0x00;
   cobolPictureTable['B']['V'] = 0x11;
   cobolPictureTable['B']['P'] = 0x01;
   cobolPictureTable['B']['G'] = 0x11;
   cobolPictureTable['B']['N'] = 0x00;

   cobolPictureTable['C']['B'] = 0x00;
   cobolPictureTable['C'][','] = 0x00;
   cobolPictureTable['C']['0'] = 0x00;
   cobolPictureTable['C']['/'] = 0x00;
   cobolPictureTable['C']['.'] = 0x00;
   cobolPictureTable['C']['C'] = 0x00;
   cobolPictureTable['C']['D'] = 0x00;
   cobolPictureTable['C']['E'] = 0x00;
   cobolPictureTable['C']['Z'] = 0x00;
   cobolPictureTable['C']['*'] = 0x00;
   cobolPictureTable['C']['+'] = 0x00;
   cobolPictureTable['C']['-'] = 0x00;
   cobolPictureTable['C']['$'] = 0x00;
   cobolPictureTable['C']['9'] = 0x00;
   cobolPictureTable['C']['A'] = 0x00;
   cobolPictureTable['C']['X'] = 0x00;
   cobolPictureTable['C']['S'] = 0x00;
   cobolPictureTable['C']['V'] = 0x00;
   cobolPictureTable['C']['P'] = 0x00;
   cobolPictureTable['C']['G'] = 0x00;
   cobolPictureTable['C']['N'] = 0x00;

   cobolPictureTable['D']['B'] = 0x00;
   cobolPictureTable['D'][','] = 0x00;
   cobolPictureTable['D']['0'] = 0x00;
   cobolPictureTable['D']['/'] = 0x00;
   cobolPictureTable['D']['.'] = 0x00;
   cobolPictureTable['D']['C'] = 0x00;
   cobolPictureTable['D']['D'] = 0x00;
   cobolPictureTable['D']['E'] = 0x00;
   cobolPictureTable['D']['Z'] = 0x00;
   cobolPictureTable['D']['*'] = 0x00;
   cobolPictureTable['D']['+'] = 0x00;
   cobolPictureTable['D']['-'] = 0x00;
   cobolPictureTable['D']['$'] = 0x00;
   cobolPictureTable['D']['9'] = 0x00;
   cobolPictureTable['D']['A'] = 0x00;
   cobolPictureTable['D']['X'] = 0x00;
   cobolPictureTable['D']['S'] = 0x00;
   cobolPictureTable['D']['V'] = 0x00;
   cobolPictureTable['D']['P'] = 0x00;
   cobolPictureTable['D']['G'] = 0x00;
   cobolPictureTable['D']['N'] = 0x00;

   cobolPictureTable['0']['B'] = 0x11;
   cobolPictureTable['0'][','] = 0x11;
   cobolPictureTable['0']['0'] = 0x11;
   cobolPictureTable['0']['/'] = 0x11;
   cobolPictureTable['0']['.'] = 0x11;
   cobolPictureTable['0']['C'] = 0x11;
   cobolPictureTable['0']['D'] = 0x11;
   cobolPictureTable['0']['E'] = 0x00;
   cobolPictureTable['0']['Z'] = 0x11;
   cobolPictureTable['0']['*'] = 0x11;
   cobolPictureTable['0']['+'] = 0x11;
   cobolPictureTable['0']['-'] = 0x11;
   cobolPictureTable['0']['$'] = 0x11;
   cobolPictureTable['0']['9'] = 0x11;
   cobolPictureTable['0']['A'] = 0x11;
   cobolPictureTable['0']['X'] = 0x11;
   cobolPictureTable['0']['S'] = 0x00;
   cobolPictureTable['0']['V'] = 0x11;
   cobolPictureTable['0']['P'] = 0x01;
   cobolPictureTable['0']['G'] = 0x00;
   cobolPictureTable['0']['N'] = 0x00;

   cobolPictureTable['/']['B'] = 0x11;
   cobolPictureTable['/'][','] = 0x11;
   cobolPictureTable['/']['0'] = 0x11;
   cobolPictureTable['/']['/'] = 0x11;
   cobolPictureTable['/']['.'] = 0x11;
   cobolPictureTable['/']['C'] = 0x11;
   cobolPictureTable['/']['D'] = 0x11;
   cobolPictureTable['/']['E'] = 0x00;
   cobolPictureTable['/']['Z'] = 0x11;
   cobolPictureTable['/']['*'] = 0x11;
   cobolPictureTable['/']['+'] = 0x11;
   cobolPictureTable['/']['-'] = 0x11;
   cobolPictureTable['/']['$'] = 0x11;
   cobolPictureTable['/']['9'] = 0x11;
   cobolPictureTable['/']['A'] = 0x11;
   cobolPictureTable['/']['X'] = 0x11;
   cobolPictureTable['/']['S'] = 0x00;
   cobolPictureTable['/']['V'] = 0x11;
   cobolPictureTable['/']['P'] = 0x01;
   cobolPictureTable['/']['G'] = 0x00;
   cobolPictureTable['/']['N'] = 0x00;

   cobolPictureTable[',']['B'] = 0x11;
   cobolPictureTable[','][','] = 0x11;
   cobolPictureTable[',']['0'] = 0x11;
   cobolPictureTable[',']['/'] = 0x11;
   cobolPictureTable[',']['.'] = 0x11;
   cobolPictureTable[',']['C'] = 0x11;
   cobolPictureTable[',']['D'] = 0x11;
   cobolPictureTable[',']['E'] = 0x11;
   cobolPictureTable[',']['Z'] = 0x11;
   cobolPictureTable[',']['*'] = 0x11;
   cobolPictureTable[',']['+'] = 0x11;
   cobolPictureTable[',']['-'] = 0x11;
   cobolPictureTable[',']['$'] = 0x11;
   cobolPictureTable[',']['9'] = 0x11;
   cobolPictureTable[',']['A'] = 0x00;
   cobolPictureTable[',']['X'] = 0x00;
   cobolPictureTable[',']['S'] = 0x00;
   cobolPictureTable[',']['V'] = 0x11;
   cobolPictureTable[',']['P'] = 0x01;
   cobolPictureTable[',']['G'] = 0x00;
   cobolPictureTable[',']['N'] = 0x00;

   cobolPictureTable['.']['B'] = 0x11;
   cobolPictureTable['.'][','] = 0x11;
   cobolPictureTable['.']['0'] = 0x11;
   cobolPictureTable['.']['/'] = 0x11;
   cobolPictureTable['.']['.'] = 0x00;
   cobolPictureTable['.']['C'] = 0x11;
   cobolPictureTable['.']['D'] = 0x11;
   cobolPictureTable['.']['E'] = 0x11;
   cobolPictureTable['.']['Z'] = 0x10;
   cobolPictureTable['.']['*'] = 0x10;
   cobolPictureTable['.']['+'] = 0x10;
   cobolPictureTable['.']['-'] = 0x10;
   cobolPictureTable['.']['$'] = 0x10;
   cobolPictureTable['.']['9'] = 0x11;
   cobolPictureTable['.']['A'] = 0x00;
   cobolPictureTable['.']['X'] = 0x00;
   cobolPictureTable['.']['S'] = 0x00;
   cobolPictureTable['.']['V'] = 0x00;
   cobolPictureTable['.']['P'] = 0x00;
   cobolPictureTable['.']['G'] = 0x00;
   cobolPictureTable['.']['N'] = 0x00;

   cobolPictureTable['+']['B'] = 0x11;
   cobolPictureTable['+'][','] = 0x11;
   cobolPictureTable['+']['0'] = 0x11;
   cobolPictureTable['+']['/'] = 0x11;
   cobolPictureTable['+']['.'] = 0x11;
   cobolPictureTable['+']['C'] = 0x00;
   cobolPictureTable['+']['D'] = 0x00;
   cobolPictureTable['+']['E'] = 0x00;
   cobolPictureTable['+']['Z'] = 0x11;
   cobolPictureTable['+']['*'] = 0x11;
   cobolPictureTable['+']['+'] = 0x11;
   cobolPictureTable['+']['-'] = 0x00;
   cobolPictureTable['+']['$'] = 0x11;
   cobolPictureTable['+']['9'] = 0x11;
   cobolPictureTable['+']['A'] = 0x00;
   cobolPictureTable['+']['X'] = 0x00;
   cobolPictureTable['+']['S'] = 0x00;
   cobolPictureTable['+']['V'] = 0x11;
   cobolPictureTable['+']['P'] = 0x11;
   cobolPictureTable['+']['G'] = 0x00;
   cobolPictureTable['+']['N'] = 0x00;

   cobolPictureTable['-']['B'] = 0x11;
   cobolPictureTable['-'][','] = 0x11;
   cobolPictureTable['-']['0'] = 0x11;
   cobolPictureTable['-']['/'] = 0x11;
   cobolPictureTable['-']['.'] = 0x11;
   cobolPictureTable['-']['C'] = 0x00;
   cobolPictureTable['-']['D'] = 0x00;
   cobolPictureTable['-']['E'] = 0x00;
   cobolPictureTable['-']['Z'] = 0x11;
   cobolPictureTable['-']['*'] = 0x11;
   cobolPictureTable['-']['+'] = 0x00;
   cobolPictureTable['-']['-'] = 0x11;
   cobolPictureTable['-']['$'] = 0x11;
   cobolPictureTable['-']['9'] = 0x11;
   cobolPictureTable['-']['A'] = 0x00;
   cobolPictureTable['-']['X'] = 0x00;
   cobolPictureTable['-']['S'] = 0x00;
   cobolPictureTable['-']['V'] = 0x11;
   cobolPictureTable['-']['P'] = 0x11;
   cobolPictureTable['-']['G'] = 0x00;
   cobolPictureTable['-']['N'] = 0x00;

   cobolPictureTable['E']['B'] = 0x00;
   cobolPictureTable['E'][','] = 0x00;
   cobolPictureTable['E']['0'] = 0x00;
   cobolPictureTable['E']['/'] = 0x00;
   cobolPictureTable['E']['.'] = 0x00;
   cobolPictureTable['E']['C'] = 0x00;
   cobolPictureTable['E']['D'] = 0x00;
   cobolPictureTable['E']['E'] = 0x00;
   cobolPictureTable['E']['Z'] = 0x00;
   cobolPictureTable['E']['*'] = 0x00;
   cobolPictureTable['E']['+'] = 0x10;
   cobolPictureTable['E']['-'] = 0x10;
   cobolPictureTable['E']['$'] = 0x00;
   cobolPictureTable['E']['9'] = 0x00;
   cobolPictureTable['E']['A'] = 0x00;
   cobolPictureTable['E']['X'] = 0x00;
   cobolPictureTable['E']['S'] = 0x00;
   cobolPictureTable['E']['V'] = 0x00;
   cobolPictureTable['E']['P'] = 0x00;
   cobolPictureTable['E']['G'] = 0x00;
   cobolPictureTable['E']['N'] = 0x00;

   cobolPictureTable['Z']['B'] = 0x11;
   cobolPictureTable['Z'][','] = 0x11;
   cobolPictureTable['Z']['0'] = 0x11;
   cobolPictureTable['Z']['/'] = 0x11;
   cobolPictureTable['Z']['.'] = 0x11;
   cobolPictureTable['Z']['C'] = 0x11;
   cobolPictureTable['Z']['D'] = 0x11;
   cobolPictureTable['Z']['E'] = 0x00;
   cobolPictureTable['Z']['Z'] = 0x11;
   cobolPictureTable['Z']['*'] = 0x00;
   cobolPictureTable['Z']['+'] = 0x11;
   cobolPictureTable['Z']['-'] = 0x11;
   cobolPictureTable['Z']['$'] = 0x00;
   cobolPictureTable['Z']['9'] = 0x11;
   cobolPictureTable['Z']['A'] = 0x00;
   cobolPictureTable['Z']['X'] = 0x00;
   cobolPictureTable['Z']['S'] = 0x00;
   cobolPictureTable['Z']['V'] = 0x11;
   cobolPictureTable['Z']['P'] = 0x01;
   cobolPictureTable['Z']['G'] = 0x00;
   cobolPictureTable['Z']['N'] = 0x00;

   cobolPictureTable['*']['B'] = 0x11;
   cobolPictureTable['*'][','] = 0x11;
   cobolPictureTable['*']['0'] = 0x11;
   cobolPictureTable['*']['/'] = 0x11;
   cobolPictureTable['*']['.'] = 0x11;
   cobolPictureTable['*']['C'] = 0x11;
   cobolPictureTable['*']['D'] = 0x11;
   cobolPictureTable['*']['E'] = 0x00;
   cobolPictureTable['*']['Z'] = 0x00;
   cobolPictureTable['*']['*'] = 0x11;
   cobolPictureTable['*']['+'] = 0x11;
   cobolPictureTable['*']['-'] = 0x11;
   cobolPictureTable['*']['$'] = 0x00;
   cobolPictureTable['*']['9'] = 0x11;
   cobolPictureTable['*']['A'] = 0x00;
   cobolPictureTable['*']['X'] = 0x00;
   cobolPictureTable['*']['S'] = 0x00;
   cobolPictureTable['*']['V'] = 0x11;
   cobolPictureTable['*']['P'] = 0x01;
   cobolPictureTable['*']['G'] = 0x00;
   cobolPictureTable['*']['N'] = 0x00;

   cobolPictureTable['$']['B'] = 0x11;
   cobolPictureTable['$'][','] = 0x11;
   cobolPictureTable['$']['0'] = 0x11;
   cobolPictureTable['$']['/'] = 0x11;
   cobolPictureTable['$']['.'] = 0x11;
   cobolPictureTable['$']['C'] = 0x11;
   cobolPictureTable['$']['D'] = 0x11;
   cobolPictureTable['$']['E'] = 0x00;
   cobolPictureTable['$']['Z'] = 0x11;
   cobolPictureTable['$']['*'] = 0x11;
   cobolPictureTable['$']['+'] = 0x11;
   cobolPictureTable['$']['-'] = 0x11;
   cobolPictureTable['$']['$'] = 0x11;
   cobolPictureTable['$']['9'] = 0x11;
   cobolPictureTable['$']['A'] = 0x00;
   cobolPictureTable['$']['X'] = 0x00;
   cobolPictureTable['$']['S'] = 0x00;
   cobolPictureTable['$']['V'] = 0x11;
   cobolPictureTable['$']['P'] = 0x11;
   cobolPictureTable['$']['G'] = 0x00;
   cobolPictureTable['$']['N'] = 0x00;

   cobolPictureTable['9']['B'] = 0x11;
   cobolPictureTable['9'][','] = 0x11;
   cobolPictureTable['9']['0'] = 0x11;
   cobolPictureTable['9']['/'] = 0x11;
   cobolPictureTable['9']['.'] = 0x11;
   cobolPictureTable['9']['C'] = 0x11;
   cobolPictureTable['9']['D'] = 0x11;
   cobolPictureTable['9']['E'] = 0x11;
   cobolPictureTable['9']['Z'] = 0x00;
   cobolPictureTable['9']['*'] = 0x00;
   cobolPictureTable['9']['+'] = 0x11;
   cobolPictureTable['9']['-'] = 0x11;
   cobolPictureTable['9']['$'] = 0x00;
   cobolPictureTable['9']['9'] = 0x11;
   cobolPictureTable['9']['A'] = 0x11;
   cobolPictureTable['9']['X'] = 0x11;
   cobolPictureTable['9']['S'] = 0x00;
   cobolPictureTable['9']['V'] = 0x11;
   cobolPictureTable['9']['P'] = 0x01;
   cobolPictureTable['9']['G'] = 0x00;
   cobolPictureTable['9']['N'] = 0x00;

   cobolPictureTable['A']['B'] = 0x11;
   cobolPictureTable['A'][','] = 0x11;
   cobolPictureTable['A']['0'] = 0x11;
   cobolPictureTable['A']['/'] = 0x11;
   cobolPictureTable['A']['.'] = 0x00;
   cobolPictureTable['A']['C'] = 0x00;
   cobolPictureTable['A']['D'] = 0x00;
   cobolPictureTable['A']['E'] = 0x00;
   cobolPictureTable['A']['Z'] = 0x00;
   cobolPictureTable['A']['*'] = 0x00;
   cobolPictureTable['A']['+'] = 0x00;
   cobolPictureTable['A']['-'] = 0x00;
   cobolPictureTable['A']['$'] = 0x00;
   cobolPictureTable['A']['9'] = 0x11;
   cobolPictureTable['A']['A'] = 0x11;
   cobolPictureTable['A']['X'] = 0x11;
   cobolPictureTable['A']['S'] = 0x00;
   cobolPictureTable['A']['V'] = 0x00;
   cobolPictureTable['A']['P'] = 0x00;
   cobolPictureTable['A']['G'] = 0x00;
   cobolPictureTable['A']['N'] = 0x00;

   cobolPictureTable['X']['B'] = 0x11;
   cobolPictureTable['X'][','] = 0x11;
   cobolPictureTable['X']['0'] = 0x11;
   cobolPictureTable['X']['/'] = 0x11;
   cobolPictureTable['X']['.'] = 0x00;
   cobolPictureTable['X']['C'] = 0x00;
   cobolPictureTable['X']['D'] = 0x00;
   cobolPictureTable['X']['E'] = 0x00;
   cobolPictureTable['X']['Z'] = 0x00;
   cobolPictureTable['X']['*'] = 0x00;
   cobolPictureTable['X']['+'] = 0x00;
   cobolPictureTable['X']['-'] = 0x00;
   cobolPictureTable['X']['$'] = 0x00;
   cobolPictureTable['X']['9'] = 0x11;
   cobolPictureTable['X']['A'] = 0x11;
   cobolPictureTable['X']['X'] = 0x11;
   cobolPictureTable['X']['S'] = 0x00;
   cobolPictureTable['X']['V'] = 0x00;
   cobolPictureTable['X']['P'] = 0x00;
   cobolPictureTable['X']['G'] = 0x00;
   cobolPictureTable['X']['N'] = 0x00;

   cobolPictureTable['S']['B'] = 0x00;
   cobolPictureTable['S'][','] = 0x00;
   cobolPictureTable['S']['0'] = 0x00;
   cobolPictureTable['S']['/'] = 0x00;
   cobolPictureTable['S']['.'] = 0x00;
   cobolPictureTable['S']['C'] = 0x00;
   cobolPictureTable['S']['D'] = 0x00;
   cobolPictureTable['S']['E'] = 0x00;
   cobolPictureTable['S']['Z'] = 0x00;
   cobolPictureTable['S']['*'] = 0x00;
   cobolPictureTable['S']['+'] = 0x00;
   cobolPictureTable['S']['-'] = 0x00;
   cobolPictureTable['S']['$'] = 0x00;
   cobolPictureTable['S']['9'] = 0x11;
   cobolPictureTable['S']['A'] = 0x00;
   cobolPictureTable['S']['X'] = 0x00;
   cobolPictureTable['S']['S'] = 0x00;
   cobolPictureTable['S']['V'] = 0x11;
   cobolPictureTable['S']['P'] = 0x11;
   cobolPictureTable['S']['G'] = 0x00;
   cobolPictureTable['S']['N'] = 0x00;

   cobolPictureTable['V']['B'] = 0x11;
   cobolPictureTable['V'][','] = 0x11;
   cobolPictureTable['V']['0'] = 0x11;
   cobolPictureTable['V']['/'] = 0x11;
   cobolPictureTable['V']['.'] = 0x00;
   cobolPictureTable['V']['C'] = 0x11;
   cobolPictureTable['V']['D'] = 0x11;
   cobolPictureTable['V']['E'] = 0x11;
   cobolPictureTable['V']['Z'] = 0x10;
   cobolPictureTable['V']['*'] = 0x10;
   cobolPictureTable['V']['+'] = 0x10;
   cobolPictureTable['V']['-'] = 0x10;
   cobolPictureTable['V']['$'] = 0x10;
   cobolPictureTable['V']['9'] = 0x11;
   cobolPictureTable['V']['A'] = 0x00;
   cobolPictureTable['V']['X'] = 0x00;
   cobolPictureTable['V']['S'] = 0x00;
   cobolPictureTable['V']['V'] = 0x00;
   cobolPictureTable['V']['P'] = 0x10;
   cobolPictureTable['V']['G'] = 0x00;
   cobolPictureTable['V']['N'] = 0x00;

   cobolPictureTable['P']['B'] = 0x11;
   cobolPictureTable['P'][','] = 0x11;
   cobolPictureTable['P']['0'] = 0x11;
   cobolPictureTable['P']['/'] = 0x11;
   cobolPictureTable['P']['.'] = 0x00;
   cobolPictureTable['P']['C'] = 0x11;
   cobolPictureTable['P']['D'] = 0x11;
   cobolPictureTable['P']['E'] = 0x00;
   cobolPictureTable['P']['Z'] = 0x10;
   cobolPictureTable['P']['*'] = 0x10;
   cobolPictureTable['P']['+'] = 0x10;
   cobolPictureTable['P']['-'] = 0x10;
   cobolPictureTable['P']['$'] = 0x10;
   cobolPictureTable['P']['9'] = 0x10;
   cobolPictureTable['P']['A'] = 0x00;
   cobolPictureTable['P']['X'] = 0x00;
   cobolPictureTable['P']['S'] = 0x00;
   cobolPictureTable['P']['V'] = 0x11;
   cobolPictureTable['P']['P'] = 0x11;
   cobolPictureTable['P']['G'] = 0x00;
   cobolPictureTable['P']['N'] = 0x00;

   cobolPictureTable['G']['B'] = 0x11;
   cobolPictureTable['G'][','] = 0x00;
   cobolPictureTable['G']['0'] = 0x00;
   cobolPictureTable['G']['/'] = 0x00;
   cobolPictureTable['G']['.'] = 0x00;
   cobolPictureTable['G']['C'] = 0x00;
   cobolPictureTable['G']['D'] = 0x00;
   cobolPictureTable['G']['E'] = 0x00;
   cobolPictureTable['G']['Z'] = 0x00;
   cobolPictureTable['G']['*'] = 0x00;
   cobolPictureTable['G']['+'] = 0x00;
   cobolPictureTable['G']['-'] = 0x00;
   cobolPictureTable['G']['$'] = 0x00;
   cobolPictureTable['G']['9'] = 0x00;
   cobolPictureTable['G']['A'] = 0x00;
   cobolPictureTable['G']['X'] = 0x00;
   cobolPictureTable['G']['S'] = 0x00;
   cobolPictureTable['G']['V'] = 0x00;
   cobolPictureTable['G']['P'] = 0x00;
   cobolPictureTable['G']['G'] = 0x11;
   cobolPictureTable['G']['N'] = 0x00;

   cobolPictureTable['N']['B'] = 0x00;
   cobolPictureTable['N'][','] = 0x00;
   cobolPictureTable['N']['0'] = 0x00;
   cobolPictureTable['N']['/'] = 0x00;
   cobolPictureTable['N']['.'] = 0x00;
   cobolPictureTable['N']['C'] = 0x00;
   cobolPictureTable['N']['D'] = 0x00;
   cobolPictureTable['N']['E'] = 0x00;
   cobolPictureTable['N']['Z'] = 0x00;
   cobolPictureTable['N']['*'] = 0x00;
   cobolPictureTable['N']['+'] = 0x00;
   cobolPictureTable['N']['-'] = 0x00;
   cobolPictureTable['N']['$'] = 0x00;
   cobolPictureTable['N']['9'] = 0x00;
   cobolPictureTable['N']['A'] = 0x00;
   cobolPictureTable['N']['X'] = 0x00;
   cobolPictureTable['N']['S'] = 0x00;
   cobolPictureTable['N']['V'] = 0x00;
   cobolPictureTable['N']['P'] = 0x00;
   cobolPictureTable['N']['G'] = 0x00;
   cobolPictureTable['N']['N'] = 0x11;

  }

/**
 * Get the number of replications of a cobol type (eg. P(99))
 * @param picString String - a string containing a PIC field
 * @param index int - offset into the string
 * @return int - number of replications
 */

private static int getNumberOfReps (String picString, int index )
   {
   int i;

   for (i=index;
           i < picString.length() && Character.isDigit(picString.charAt(i));
           i++);

   i = i - index;

   if ( i == 0 )
   {
        // expected digits - non digits found
        // raise exception
		return -1;
   }

   StringBuffer sb = new StringBuffer(i);
   sb.append(picString.toCharArray(),index,i);
   return ( Integer.parseInt(sb.toString()) );
  }
/**
 * The method parses the pic string.
 * @param inpicString String - a PIC string
 * @exception RecordException
 * The inpicString is an invalid PIC representation
 */

   public void pic(String inpicString)
   {
         if ( inpicString == null )
           return;

         // Check that the picture string is not too long (> 50)
         if ( inpicString.length() > 50 )
         {
            Object[] obj = {inpicString.toUpperCase(java.util.Locale.US), cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
            yyerror(CobolResource.getString("_EXC_Pic_string_length_exceeds_50", obj), true);
            return;
         }

         picString = inpicString.toUpperCase(java.util.Locale.US);

         StringBuffer expandedPicBuf = new StringBuffer(0);
         int[] occurrances = new int[256];
         int decimal_indicator = 0x01;
         int toIndex = 0;

         if (DEBUG)
         {
           System.out.println("picString.length() " + picString.length());
           System.out.println("picString:" + "|" + picString + "|");
         }

         for ( int fromIndex=0; fromIndex < picString.length(); fromIndex++)
         {

           int repetitions = 1;
           if (DEBUG)
                System.out.println("fromIndex " + fromIndex);

           // Increment the number of occurrances of character
           occurrances[ (int) picString.charAt(fromIndex) ]++;

           toIndex = expandedPicBuf.length();
           if (DEBUG)
                System.out.println("toIndex length : " + toIndex);

           // See if the transition to this character is valid
           if (toIndex > 1 )
           {
                if ( (expandedPicBuf.charAt(toIndex -2) == 'C' &&
                         expandedPicBuf.charAt(toIndex -1) == 'R' ) ||
                        (expandedPicBuf.charAt(toIndex -2) == 'D' &&
                         expandedPicBuf.charAt(toIndex -1) == 'B') )
                {
                   if ( (cobolPictureTable[expandedPicBuf.charAt(toIndex-2)][picString.charAt(fromIndex)]
                                        & decimal_indicator) == 0x00 )
// XXJS                    & decimal_indicator) != 0x00 )
                   {
                      Object[] obj = {cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
                      yyerror(CobolResource.getString("_EXC_Pic_string_is_invalid", obj), true);
                      return;
                   }

                }
           }
           if ( toIndex > 0 )
           {
                if ( (cobolPictureTable[expandedPicBuf.charAt(toIndex-1)][picString.charAt(fromIndex)]
                                 & decimal_indicator) == 0x00 )
// XXJS                 & decimal_indicator) != 0x00 )
                   {
                      Object[] obj = {cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
                      yyerror(CobolResource.getString("_EXC_Pic_string_is_invalid", obj), true);
                      return;
                   }

           }

           // Detect numeric string

           if (picString.charAt(fromIndex) != 'S' &&
                  picString.charAt(fromIndex) != 'V' &&
                  picString.charAt(fromIndex) != '9' &&
                  picString.charAt(fromIndex) != 'P' )
           {
                  numeric=false;
           }
           // Check for decimal positions - implied decimal dealt with later

           if ( picString.charAt(fromIndex) == '.' ||
                   picString.charAt(fromIndex) == 'V' )
           {
                decimal = true;
                decimalPosition = toIndex;
                decimal_indicator = 0x10;
           }


           switch ( picString.charAt(fromIndex) )
           {
   // 1.0.1 NL changes
           case 'N':
           case 'G':
                  inDBCS = true;
   // 1.0.1 NL end
           case 'A':
           case 'B':
           case 'P':
           case 'X':
           case 'Z':
           case '9':
           case '0':
           case '/':
           case ',':
           case '+':
           case '-':
           case '*':
           case '$':


                if ( (picString.length() > fromIndex +1) &&
                   (picString.charAt(fromIndex+1) == '('))
                  {
                   // Get the integer number - eg. X(300)
                   repetitions = getNumberOfReps(picString, (fromIndex+2) );
                   if(repetitions == -1) {
						Object[] obj = {picString, cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
						yyerror(CobolResource.getString("_EXC_Pic_string_only_digits", obj), true);
                   		return;
                   }

                   // check for limit (MAX_STRING_LENGTH) - no limit on
                   // A or X

                   int totalLength = expandedPicBuf.length() + repetitions;
                   if ( (totalLength > MAX_STRING_LENGTH) &&
   // 1.0.1 NL changes
                           (picString.charAt(fromIndex) != 'G') &&
                           (picString.charAt(fromIndex) != 'N') &&
   // 1.0.1 NL end
                           (picString.charAt(fromIndex) != 'A') &&
                           (picString.charAt(fromIndex) != 'X') )
                   {
                      Object[] obj = {inpicString, cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
                      yyerror(CobolResource.getString("_EXC_Pic_string_too_long", obj), true);
                      return;
                   }

                   // copy the repetitions to the expanded buffer
                   for (int r=0; r < repetitions; r++)
                   {
                        expandedPicBuf.append(picString.charAt(fromIndex));
                   }
                   toIndex = toIndex + repetitions;


                   // Increment the number of occurrances.  eg X(1), X(100)

                   occurrances[ (int) picString.charAt(fromIndex) ] =
                        occurrances[(int)picString.charAt(fromIndex)] + (repetitions -1);


                   if( picString.charAt(fromIndex) == 'P' )
                   {
                         decimal = true;
                         decimalPosition = expandedPicBuf.length() -1;
                         decimal_indicator = 0x10;
                         // Account for scale for implied decimal
                         scale = scale + repetitions;
                   }
                   else if ( picString.charAt(fromIndex) == '9' )
                   {
                         numberOfNines = numberOfNines + repetitions;
                   }

                  // Go to the bracketed item ")"
                  // fromIndex+1 == "("  fromIndex+2 == digit

                   fromIndex= fromIndex +2;
                   for (; picString.charAt(fromIndex) != ')'; fromIndex++)
                   {
                        if ( !Character.isDigit(picString.charAt(fromIndex)) )
                        {
                           Object[] obj = {inpicString, cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
                           yyerror(CobolResource.getString("_EXC_Pic_string_only_digits", obj), true);
                           return;
                        }
                   }

                } // fromIndex+1 != "("
                else
                {
                   // fromIndex+2 != "(" so only one occurrance of character
                   expandedPicBuf.append(picString.charAt(fromIndex));
                   toIndex++;

                   if( picString.charAt(fromIndex) == 'P' )
                   {
                         decimal = true;
                         decimalPosition = expandedPicBuf.length() -1;
                         decimal_indicator = 0x10;
                         // Account for scale for implied decimal
                         scale++;
                   }
                   else if ( picString.charAt(fromIndex) == '9' )
                   {
                         numberOfNines++;
                   }
                }

                break;

           case '.':       // Characters not replicable
           case 'S':
           case 'V':
           case 'E':


                if ( occurrances[(int) picString.charAt(fromIndex)] > 1 )
                {
                   Object[] obj = {inpicString, cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
                   yyerror(CobolResource.getString("_EXC_Pic_string_one_occurrence", obj), true);
                   return;
                }

                expandedPicBuf.append(picString.charAt(fromIndex));
                toIndex++;
                break;

           case 'C':  // Handle double characters   'CR'

                if ( occurrances[(int) picString.charAt(fromIndex)] > 1 )
                {
                   Object[] obj = {inpicString, cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
                   yyerror(CobolResource.getString("_EXC_Pic_string_only_digits", obj), true);
                   return;
                }

                if ( picString.charAt(fromIndex+1) != 'R' )
                {
                   Object[] obj = {inpicString, cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
                   yyerror(CobolResource.getString("_EXC_Pic_string_r_must_follow_c", obj), true);
                   return;
                }

                expandedPicBuf.append('C');
                expandedPicBuf.append('R');
                fromIndex++;  // handle two char
                toIndex = toIndex +2;
                break;

           case 'D':     //DB

                if ( occurrances[(int) picString.charAt(fromIndex)] > 1 )
                {
                   Object[] obj = {inpicString, cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
                   yyerror(CobolResource.getString("_EXC_Pic_string_only_one_d_allowed", obj), true);
                   return;
                }

                if ( picString.charAt(fromIndex+1) != 'B' )
                {
                   Object[] obj = {inpicString, cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
                   yyerror(CobolResource.getString("_EXC_Pic_string_expected_b_to_follow_d", obj), true);
                   return;
                }

                expandedPicBuf.append('D');
                expandedPicBuf.append('B');
                fromIndex++; // handle two char
                toIndex = toIndex +2;
                break;

           default:
              // not a valid character
              Object[] obj = {inpicString, cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
              yyerror(CobolResource.getString("_EXC_Pic_string_contains_invalid_characters", obj), true);
              return;

           } // END CASE

           // Ensure that number is not exceeding 31 digits or pads

           if ( (scale + numberOfNines) > 31 )
           {
              Object[] obj = {inpicString, cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
              yyerror(CobolResource.getString("_EXC_Pic_string_cannot_exceed_31_digits", obj), true);
              return;
           }

           if (DEBUG)
                System.out.println("expanded pic buf: " + expandedPicBuf);

           if ( (toIndex > MAX_STRING_LENGTH) &&
   // 1.0.1 NL changes
                   (expandedPicBuf.charAt(toIndex - 1) != 'G') &&
                   (expandedPicBuf.charAt(toIndex - 1) != 'N') &&
   // 1.0.1 NL end
                   (expandedPicBuf.charAt(toIndex - 1) != 'A') &&
                   (expandedPicBuf.charAt(toIndex - 1) != 'X') )
           {
              Object[] obj = {inpicString, Integer.toString(CobolParser.MAX_STRING_LENGTH), cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
              yyerror(CobolResource.getString("_EXC_Pic_string_cannot_exceed_characters", obj), true);
              return;
           }

         } // END FOR LOOP

         // Check whether or not picture string is valid

         if ( !( occurrances[(int)'A'] > 0 ||
                   occurrances[(int)'X'] > 0 ||
   // 1.0.1 NL changes
                   occurrances[(int)'G'] > 0 ||
                   occurrances[(int)'N'] > 0 ||
   // 1.0.1 NL end
                   occurrances[(int)'Z'] > 0 ||
                   occurrances[(int)'9'] > 0 ||
                   occurrances[(int)'*'] > 0 ||
                   ( occurrances[(int)'-'] > 1 ||
                   occurrances[(int)'+'] > 1 ||
                   occurrances[(int)'$'] > 1  )) )
         {
              Object[] obj = {cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
              yyerror(CobolResource.getString("_EXC_Pic_string_is_invalid", obj), true);
              return;
         }

         // set the sign field

         if ( expandedPicBuf.charAt(0) == 'S')
           isSign = true;

         if ( occurrances[(int)'V'] == 1)
         {
           // Don't count implied decimal position
           picFieldLength = expandedPicBuf.length() -1;
         }
         else
           picFieldLength = expandedPicBuf.length();

   // 1.0.1 NL changes
   if(inDBCS)
           picFieldLength *= 2;
   // 1.0.1 NL end

         decimalPosition = decimalPosition - expandedPicBuf.length() +1;
         if ( decimalPosition < 0 )
         {
           scale = scale * (-1);
         }

         // See if the field requires padding
         if ( occurrances[(int)'A'] > 0 ||
              occurrances[(int)'G'] > 0 ||
              occurrances[(int)'X'] > 0 ||
              occurrances[(int)'N'] > 0 )
         {
            paddingRequired = true;
         }

         if (DEBUG)
           System.out.println("Exiting updateCicsDataDescriptor");
   }

   private Vector symbols = new Vector();
   private Vector replaceList = new Vector();
   private Vector replaceByList = new Vector();
   private Vector operand1 = new Vector();
   private Vector operand2 = new Vector();
   private Stack cobolFileList = new Stack();
   private Stack tokenizerList = new Stack();
   private Stack groupStack = new Stack();
   private CircularBuffer buffer = new CircularBuffer();
   private CobolRecord pNewRecord = null;
   private boolean errorsOccurred = false;
   private short usage;
   private short sign;
   private int token = -1;
   private int replaceSize = 0;
   private int level;
   private int bound;
   private int lowBound; /*26260 GIM Save lower bound for ODO */
   private int isLowBound; /*26260 GIM Save lower bound for ODO */
   private int fillerGen;
   private int numberOfNines;
   private int decimalPosition;
   private int scale;
   private int picFieldLength;
   private boolean align;
   private boolean fill;
   private boolean paddingRequired;
   // 1.0.1 NL changes
   boolean inDBCS;
   // 1.0.1 NL end
   private boolean isSign;
   private boolean decimal;
   private boolean numeric;
   private String name = new String();
   private String value;
   private String figConst;
   private String lit;
   private String firstLit;
   private Vector possibleValues;
   private String redefinedItem;
   private String picString;
   private String javaType;
   private String previousType;
   private Vector idVector;
   private static final boolean DEBUG = false;
   private static final int MAX_STRING_LENGTH = 249;
   private static int countForIMS=0;
   private String IMSTranCode = new String();
   private static boolean bCopyContext = false;    /*16494 Flag to indicate old semantics for OF/IN in COPY statement*/
   /*-----------------------------------------------------------------*
   * Cobol picture table - describes valid PICTURES                  *
   *-----------------------------------------------------------------*/
   private static char[][] cobolPictureTable = new char[256][256];

   static {cobolPictureTableInit();}

   private static String copyright = "(c) Copyright IBM Corporation 1998, 1999.";

   List errorList = new ArrayList();
   
   /*GIM SVL CMVC: 36692 Handle unsupported types*/ 
   private Vector unsupportedTypes = new Vector();
   public Vector getUnsupportedTypes() {
   	return unsupportedTypes;
   }
   
   public List getErrorList() {
      return errorList;
   }
//#line 1950 "CobolParser.java"
//###############################################################
// method: yylexdebug : check lexer state
//###############################################################
void yylexdebug(int state,int ch)
{
String s=null;
  if (ch < 0) ch=0;
  if (ch <= YYMAXTOKEN) //check index bounds
     s = yyname[ch];    //now get it
  if (s==null)
    s = "illegal-symbol";
  debug("state "+state+", reading "+ch+" ("+s+")");
}
//###############################################################
// method: yyparse : parse input and execute indicated items
//###############################################################
int yyparse() throws CobolException
{
int yyn;       //next next thing to do
int yym;       //
int yystate;   //current parsing state from state table
String yys;    //current token string
boolean doaction;
  init_stacks();
  yynerrs = 0;
  yyerrflag = 0;
  yychar = -1;          //impossible char forces a read
  yystate=0;            //initial state
  state_push(yystate);  //save it
  while (true) //until parsing is done, either correctly, or w/error
    {
    doaction=true;
    if (yydebug) debug("loop"); 
    //#### NEXT ACTION (from reduction table)
    for (yyn=yydefred[yystate];yyn==0;yyn=yydefred[yystate])
      {
      if (yydebug) debug("yyn:"+yyn+"  state:"+yystate+"  char:"+yychar);
      if (yychar < 0)      //we want a char?
        {
        yychar = yylex();  //get next token
        //#### ERROR CHECK ####
        if (yychar < 0)    //it it didn't work/error
          {
          yychar = 0;      //change it to default string (no -1!)
          if (yydebug)
            yylexdebug(yystate,yychar);
          }
        }//yychar<0
      yyn = yysindex[yystate];  //get amount to shift by (shift index)
      if ((yyn != 0) && (yyn += yychar) >= 0 &&
          yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
        {
        if (yydebug)
          debug("state "+yystate+", shifting to state "+yytable[yyn]+"");
        //#### NEXT STATE ####
        yystate = yytable[yyn];//we are in a new state
        state_push(yystate);   //save it
        val_push(yylval);      //push our lval as the input for next rule
        yychar = -1;           //since we have 'eaten' a token, say we need another
        if (yyerrflag > 0)     //have we recovered an error?
           --yyerrflag;        //give ourselves credit
        doaction=false;        //but don't process yet
        break;   //quit the yyn=0 loop
        }

    yyn = yyrindex[yystate];  //reduce
    if ((yyn !=0 ) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
      {   //we reduced!
      if (yydebug) debug("reduce");
      yyn = yytable[yyn];
      doaction=true; //get ready to execute
      break;         //drop down to actions
      }
    else //ERROR RECOVERY
      {
      if (yyerrflag==0)
        {
	Object[] obj ={cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
	yyerror(CobolResource.getString("_EXC_Syntax_error", obj), true);
        yynerrs++;
        }
      if (yyerrflag < 3) //low error count?
        {
        yyerrflag = 3;
        while (true)   //do until break
          {
          if (stateptr<0)   //check for under & overflow here
            {
//            yyerror("stack underflow. aborting...");  //note lower case 's'
            return 1;
            }
          yyn = yysindex[state_peek(0)];
          if ((yyn != 0) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
            if (yydebug)
              debug("state "+state_peek(0)+", error recovery shifting to state "+yytable[yyn]+" ");
            yystate = yytable[yyn];
            state_push(yystate);
            val_push(yylval);
            doaction=false;
            break;
            }
          else
            {
            if (yydebug)
              debug("error recovery discarding state "+state_peek(0)+" ");
            if (stateptr<0)   //check for under & overflow here
              {
	Object[] obj ={cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
	yyerror(CobolResource.getString("_EXC_Syntax_error", obj), true);
              return 1;
              }
            state_pop();
            val_pop();
            }
          }
        }
      else            //discard this token
        {
        if (yychar == 0)
          return 1; //yyabort
        if (yydebug)
          {
          yys = null;
          if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
          if (yys == null) yys = "illegal-symbol";
          debug("state "+yystate+", error recovery discards token "+yychar+" ("+yys+")");
          }
        yychar = -1;  //read another
        }
      }//end error recovery
    }//yyn=0 loop
    if (!doaction)   //any reason not to proceed?
      continue;      //skip action
    yym = yylen[yyn];          //get count of terminals on rhs
    if (yydebug)
      debug("state "+yystate+", reducing "+yym+" by rule "+yyn+" ("+yyrule[yyn]+")");
    yyval = val_peek(yym-1);   //get current semantic value
    switch(yyn)
      {
//########## USER-SUPPLIED ACTIONS ##########
case 4:
//#line 74 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{ initInfo(); }
break;
case 5:
//#line 76 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{ storeInfo(); }
break;
case 6:
//#line 77 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{bCopyContext = true; /*16494 Keep old semantics for OF/IN in COPY statement*/ }
break;
case 7:
//#line 79 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                           	  bCopyContext = false; /*16494 Turn on OF/IN for ODO qualifiers.*/
                              if ( tokenizerList.size() > 1 && replaceSize > 0 )
                              {
                                 Object[] obj = {cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
                             	  yyerror(CobolResource.getString("_EXC_Invalid_nested_copy_statement", obj), true);
                              }
                              else if ( tokenizerList.size() > 1 && !replaceList.isEmpty() )
                              {
		                  Object[] obj = {cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
                                 yyerror(CobolResource.getString("_EXC_Nested_copy_cannot_contain_replacing_phrase", obj), true);
                              }
                              else
                              {
                                 if ( tokenizerList.size() < 2 )
                                    replaceSize = replaceList.size();
                                 String copybook = buffer.get(val_peek(3)).toString(); /*16494 Adjust order of symbols (was $2 and $3)*/
                                 String filePath = buffer.get(val_peek(2)).toString() + System.getProperty("file.separator");
			          String copybookFullName = filePath + copybook + ".cbl";
   			          if ( !new File(copybookFullName).isFile() )
   				     copybookFullName = filePath + copybook + ".cpy";
   			          if ( !new File(copybookFullName).isFile() )
   				     copybookFullName = filePath + copybook;
			   		try {		     
					 java.net.URL copybookURL = new java.net.URL("file", null, -1, copybookFullName);
					 copybookFullName = copybookURL.toString();
			   		} catch (MalformedURLException ex) {
			   		}
   			
                                 CobolCommareaGenerator f_out = new CobolCommareaGenerator(copybookFullName);
				 f_out.generateToString();
				 cobolFileList.push(copybookFullName);
				 String str = f_out.toString();
				 if ( !replaceList.isEmpty() )
	        		 	str = f_out.replace(str, replaceList, replaceByList);
       			 tokenizerList.push(new StreamTokenizer(new StringReader(str)));
                                 replaceList = new Vector();
                                 replaceByList = new Vector();
                              }
                           }
break;
case 8:
//#line 122 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{ level = val_peek(0); }
break;
case 9:
//#line 124 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{ name = (String)buffer.get(val_peek(0)); /*Do not mangle the name*/
                             if (level!=88)
                                previousType = new String();
                           }
break;
case 10:
//#line 131 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                               yyval = buffer.put((String)buffer.get(val_peek(0)));
                               fill = false;
                           }
break;
case 11:
//#line 136 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                               yyval = buffer.put("fill_" + fillerGen);
                               fillerGen++;
                               fill = true;
                           }
break;
case 12:
//#line 142 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                               yyval = buffer.put("fill_" + fillerGen);
                               fillerGen++;
                               fill = true;
                           }
break;
case 16:
//#line 155 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{ align = true; }
break;
case 24:
//#line 166 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                              pic((String)buffer.get(val_peek(0)));

                              if ( paddingRequired )
                                 value = " ";

                              if ( usage != CobolRecord.COMP3 )
                              {
                                 if ( !numeric )
                                 {
                                    javaType = "java.lang.String";
                                 }
                                 else
                                 {
                                    if ( decimal )
                                    {
                                       javaType = "java.lang.Double";
                                    }
                                    else
                                    {
                                       if ( numberOfNines < 5 )
                                          javaType = "java.lang.Short";
                                       else if ( numberOfNines > 9 )
                                          javaType = "java.lang.Long";
                                       else
                                          javaType = "java.lang.Integer";
                                    }
                                 }
                              }
                           }
break;
case 25:
//#line 197 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                              pic(Integer.toString(val_peek(0)));

                              if ( paddingRequired )
                                 value = " ";

                              if ( usage != CobolRecord.COMP3 )
                              {
                                 if ( decimal )
                                 {
                                    javaType = "java.lang.Double";
                                 }
                                 else
                                 {
                                    if ( numberOfNines < 5 )
                                       javaType = "java.lang.Short";
                                    else if ( numberOfNines > 9 )
                                       javaType = "java.lang.Long";
                                    else
                                       javaType = "java.lang.Integer";
                                 }
                             }
                          }
break;
case 27:
//#line 226 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{ redefinedItem = (String)buffer.get(val_peek(0)); }
break;
case 28:
//#line 230 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                           if (isLowBound!=-1)
                              lowBound = val_peek(3); /*26260 GIM save this this is ODO */         
			               if (val_peek(3) > bound)
		             		   bound = val_peek(3);
			               }
break;
case 29:
//#line 239 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                             isLowBound = val_peek(2); /*26260 GIM save this if later item turns to be ODO */ 
						     if (val_peek(2) > bound)
        					 bound = val_peek(2);
                           }
break;
case 35:
//#line 254 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
			     idVector.addElement((String)buffer.get(val_peek(0))); /*16494*/ /*Do not mangle the name*/
			   }
break;
case 46:
//#line 282 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
/*			       Object[] obj = {cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};*/
/*			       yyerror(CobolResource.getString("_EXC_Hexadecimal_value_not_supported", obj), true);*/
                           }
break;
case 47:
//#line 287 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                       	   }
break;
case 48:
//#line 290 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                              value = (String)buffer.get(val_peek(0));
                              lit = value;
                           }
break;
case 49:
//#line 295 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                              figConst = buffer.get(val_peek(0)).toString();
			       if ( figConst.equals("\"") )
				  figConst = "\\" + figConst;
                               lit=figConst;
                           }
break;
case 50:
//#line 302 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                              value = "0";
                              figConst = "0";
                              lit = value;
                           }
break;
case 51:
//#line 308 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{  value = " ";
                              lit = value;}
break;
case 52:
//#line 311 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{  figConst = "\\\"";
                              lit = figConst;}
break;
case 53:
//#line 314 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{  figConst = "\'";
                              lit = figConst;}
break;
case 54:
//#line 317 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                           	  /*GIM SVL CMVC: 36900 Handle unsupported types*/ 	
                              /*Object[] obj = {cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};*/
                              /*unsupportedTypes.add(new String(CobolResource.getString("_EXC_High_value_not_supported", obj)));*/
                           }
break;
case 55:
//#line 323 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{   
                              /*GIM SVL CMVC: 36900 Handle unsupported types*/ 	
                              /*Object[] obj = {cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};*/
                              /*unsupportedTypes.add(new String(CobolResource.getString("_EXC_Low_value_not_supported", obj)));*/
                           }
break;
case 56:
//#line 329 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{   
                              /*GIM SVL CMVC: 36900 Handle unsupported types*/ 	
                              Object[] obj = {cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};                              
                              unsupportedTypes.add(new String(CobolResource.getString("_EXC_Null_value_not_supported", obj)));                              
                           }
break;
case 57:
//#line 335 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                              value = "0";
                              figConst = "0";
                              lit = value;
                           }
break;
case 58:
//#line 341 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{  value = " ";
                              lit = value;}
break;
case 59:
//#line 344 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{  figConst = "\\\"";
                              lit = figConst;}
break;
case 60:
//#line 347 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{  figConst = "\'";
                              lit = figConst;}
break;
case 61:
//#line 350 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                              /*GIM SVL CMVC: 36900 Handle unsupported types*/ 	
                             /*Object[] obj = {cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};*/
                             /*unsupportedTypes.add(new String(CobolResource.getString("_EXC_High_value_not_supported", obj)));                                                           */
                           }
break;
case 62:
//#line 356 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                              /*GIM SVL CMVC: 36900 Handle unsupported types*/ 	
                              /*Object[] obj = {cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};*/
							  /*unsupportedTypes.add(new String(CobolResource.getString("_EXC_Low_value_not_supported", obj)));                                                                                         */
                           }
break;
case 63:
//#line 362 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                              /*GIM SVL CMVC: 36900 Handle unsupported types*/ 	
                              Object[] obj = {cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
                              unsupportedTypes.add(new String(CobolResource.getString("_EXC_Null_value_not_supported", obj)));                                                           
                           }
break;
case 64:
//#line 368 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                              value = Integer.toString(val_peek(0));
                              lit = value;
                           }
break;
case 65:
//#line 375 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                              if ( level == 88 )
                              {
                                 if (previousType.equals(""))
                                 {  possibleValues.addElement((Object) lit); }
                                 else if (previousType.equals("java.lang.String"))
                                 {
                                    possibleValues.addElement(lit);
                                 }
                                 else
                                 {  if (previousType.equals("java.lang.Double"))
                                    {
                                       possibleValues.addElement(new Double(lit));
                                    }
                                    else
                                    {  if (previousType.equals("java.lang.Short"))
                                       {
                                          if (lit.startsWith("+"))
                                             lit = lit.substring(1);
                                          possibleValues.addElement(new Short(lit));
                                       }
                                       else
                                       {  if (previousType.equals("java.lang.Long"))
                                          {
                                             if (lit.startsWith("+"))
                                                lit = lit.substring(1);
                                             possibleValues.addElement(new Long(lit));
                                          }
                                          else
                                          {  if (previousType.equals("java.lang.Integer"))
                                             {
                                                if (lit.startsWith("+"))
                                                   lit = lit.substring(1);
                                                possibleValues.addElement(new Integer(lit));
                                             }
                                             else
                                             {  if (previousType.equals("java.lang.Float"))
                                                {
                                                   possibleValues.addElement(new Float(lit));
                                                }
                                                else
                                                {  if (previousType.equals("java.math.BigDecimal"))
                                                   {
                                                      if (lit.startsWith("+"))
                                                         lit = lit.substring(1);
                                                      possibleValues.addElement(new BigDecimal(lit));
                                                   }
                                                }
                                             }
                                          }
                                       }
                                    }
                                 }
                                 value=null;
                                 figConst=null;
                              }
                           }
break;
case 66:
//#line 433 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{  firstLit=lit;  }
break;
case 67:
//#line 435 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                              if ( level == 88 )
                              {
                                 if (previousType.equals("") || previousType.equals("java.lang.String"))
                                 {
                                  try {
                                    if (firstLit.length()==1 && lit.length()==1)
                                    {
                                       if (Character.isLetter(firstLit.charAt(0)) && Character.isLetter(lit.charAt(0)))
                                       {
                                          if ((Character.isLowerCase(firstLit.charAt(0)) && Character.isLowerCase(lit.charAt(0))) ||
                                              (Character.isUpperCase(firstLit.charAt(0)) && Character.isUpperCase(lit.charAt(0))))
                                          {
                                              possibleValues.addElement(new StringRange(firstLit, lit));
                                          }
                                          else
                                          {
                                             if (Character.isLowerCase(firstLit.charAt(0)))
                                             {
                                                possibleValues.addElement(new StringRange(firstLit, "z"));
                                                possibleValues.addElement(new StringRange("A", lit));
                                             }
                                             else
                                             {
                                                possibleValues.addElement(new StringRange(firstLit, "Z"));
                                                possibleValues.addElement(new StringRange("a", lit));
                                             }
                                          }
                                       }
                                       else
                                       {
                                          possibleValues.addElement(new StringRange(firstLit, lit));
                                       }
                                    }
                                    else
                                    {
                                       possibleValues.addElement(new StringRange(firstLit, lit));
                                    }
                                 }
                                 catch(IllegalArgumentException iae) {
										Object[] obj = {lit, firstLit};
										throw new CobolException(CobolResource.getString("_EXC_Invalid_String_Range", obj));                                    	
                                    }
                                 }
                                 else
                                 {  if (previousType.equals("java.lang.Double"))
                                    {
                                       possibleValues.addElement(new DoubleRange (new Double(firstLit).doubleValue(), new Double(lit).doubleValue()));
                                    }
                                    else
                                    {  if (previousType.equals("java.lang.Short"))
                                       {
                                          if (firstLit.startsWith("+"))
                                             firstLit = firstLit.substring(1);
                                          if (lit.startsWith("+"))
                                             lit = lit.substring(1);
                                          possibleValues.addElement(new ShortRange(new Short(firstLit).shortValue(), new Short(lit).shortValue()));
                                       }
                                       else
                                       {  if (previousType.equals("java.lang.Long"))
                                          {
                                             if (firstLit.startsWith("+"))
                                                firstLit = firstLit.substring(1);
                                             if (lit.startsWith("+"))
                                                lit = lit.substring(1);
                                             possibleValues.addElement(new LongRange(new Long(firstLit).longValue(), new Long(lit).longValue()));
                                          }
                                          else
                                          {  if (previousType.equals("java.lang.Integer"))
                                             {
                                                if (firstLit.startsWith("+"))
                                                   firstLit = firstLit.substring(1);
                                                if (lit.startsWith("+"))
                                                   lit = lit.substring(1);
                                                possibleValues.addElement(new IntegerRange(new Integer(firstLit).intValue(), new Integer(lit).intValue()));
                                             }
                                             else
                                             {  if (previousType.equals("java.lang.Float"))
                                                {
                                                   possibleValues.addElement(new FloatRange(new Float(firstLit).floatValue(), new Float(lit).floatValue()));
                                                }
                                                else
                                                {  if (previousType.equals("java.math.BigDecimal"))
                                                   {
                                                      if (firstLit.startsWith("+"))
                                                         firstLit = firstLit.substring(1);
                                                      if (lit.startsWith("+"))
                                                         lit = lit.substring(1);
                                                      possibleValues.addElement(new com.ibm.record.BigDecimalRange(new BigDecimal(firstLit), new BigDecimal(lit)));
                                                   }
                                                }
                                             }
                                          }
                                       }
                                    }
                                 }
                                 value = null;
                                 figConst = null;
                              }
                           }
break;
case 69:
//#line 539 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{ usage = CobolRecord.COMP; }
break;
case 70:
//#line 541 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                              usage = CobolRecord.COMP1;
                              picString = " ";
                              javaType = "java.lang.Float";
                           }
break;
case 71:
//#line 547 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                              usage = CobolRecord.COMP2;
                              picString = " ";
                              javaType = "java.lang.Double";
                           }
break;
case 72:
//#line 553 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                              usage = CobolRecord.COMP3;
                              javaType = "java.math.BigDecimal";
                           }
break;
case 73:
//#line 558 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                              usage = CobolRecord.COMP5;
/*                              Object[] obj = {cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};*/
/*                               yyerror(CobolResource.getString("_EXC_Comp5_not_supported_on_mvs", obj), false);*/
                           }
break;
case 74:
//#line 564 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{ usage = CobolRecord.DISPLAY; }
break;
case 75:
//#line 566 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                              /*GIM SVL CMVC: 36692 Handle unsupported types*/ 
                              Object[] obj = {cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
                              unsupportedTypes.add(new String(CobolResource.getString("_EXC_Compx_not_supported", obj)));
                           }
break;
case 76:
//#line 572 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                              Object[] obj = {cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
/*                            yyerror(CobolResource.getString("_EXC_Index_not_supported", obj), true);*/
                              /*GIM SVL CMVC: 36692 Handle unsupported types*/ 
                              unsupportedTypes.add(new String(CobolResource.getString("_EXC_Index_not_supported", obj)));
                           }
break;
case 77:
//#line 579 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                              /*GIM SVL CMVC: 36692 Handle unsupported types*/ 
                              Object[] obj = {cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
                              unsupportedTypes.add( new String(CobolResource.getString("_EXC_Pointer_not_supported", obj)));
                           }
break;
case 78:
//#line 585 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{ usage = CobolRecord.DISPLAY1; }
break;
case 79:
//#line 587 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{ usage = CobolRecord.NATIONAL; }
break;
case 80:
//#line 589 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
                              /*GIM SVL CMVC: 36692 Handle unsupported types*/ 
                              Object[] obj = {cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
                              unsupportedTypes.add(new String(CobolResource.getString("_EXC_Procedure_pointer_not_supported", obj)));
                           }
break;
case 81:
//#line 597 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{ sign = CobolRecord.LEADING; }
break;
case 82:
//#line 599 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{ sign = CobolRecord.LEADINGSEPARATE; }
break;
case 83:
//#line 601 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{ sign = CobolRecord.TRAILING; }
break;
case 84:
//#line 603 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{ sign = CobolRecord.TRAILINGSEPARATE; }
break;
case 90:
//#line 616 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{ yyval = buffer.put(buffer.get(val_peek(0)).toString()); }
break;
case 91:
//#line 618 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{ String parent = new File(cobolFileList.firstElement().toString()).getParent();
			     if (parent==null)
				parent = ".";
			     yyval = buffer.put(parent);
			   }
break;
case 94:
//#line 630 "C:/Rad/Rad6.0/workspace_rad/com.ibm.etools.cobol/src/cbl.y"
{
	String felement = operand1.firstElement().toString();
	String lelement = operand1.lastElement().toString();
	int findex = felement.indexOf("==");
	int lindex = lelement.lastIndexOf("==");
	if ( findex != -1 && lindex != -1 )
	{
		if ( (findex != -1 && lindex == -1) || (findex == -1 && lindex != -1) )
		{
			  Object[] obj = {cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
			  yyerror(CobolResource.getString("_EXC_Syntax_error_in_copy_statement", obj), true);
		}
		else if ( findex == lindex )
		{
			if ( felement.length() == 2 )
				operand1.removeElementAt(0);
			else
				operand1.setElementAt(felement.substring(2), 0);
			if ( lelement.length() == 2 )
				operand1.removeElementAt(operand1.size()-1);
			else
				operand1.setElementAt(lelement.substring(0, lelement.length()-2), operand1.size()-1);
		}
		else
		{
			if ( operand1.size() == 1 )
				operand1.setElementAt(felement.substring(2, lindex), 0);
			else
			{
				operand1.setElementAt(felement.substring(2), 0);
				operand1.setElementAt(lelement.substring(0, lindex), operand1.size()-1);
			}
		}
	}
	if (operand1.isEmpty() || operand1.firstElement().equals("") )
	{
		  Object[] obj = {cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
		  yyerror(CobolResource.getString("_EXC_Syntax_error_in_copy_statement", obj), true);
	}

	felement = operand2.firstElement().toString();
	lelement = operand2.lastElement().toString();
	findex = felement.indexOf("==");
	lindex = lelement.lastIndexOf("==");
	if ( findex != -1 && lindex != -1 )
	{
		if ( (findex != -1 && lindex == -1) || (findex == -1 && lindex != -1) )
		{
			  Object[] obj = {cobolFileList.peek().toString(), Integer.toString(((StreamTokenizer)tokenizerList.peek()).lineno())};
			  yyerror(CobolResource.getString("_EXC_Syntax_error_in_copy_statement", obj), true);
		}
		else if ( findex == lindex )
		{
			if ( felement.length() == 2 )
				operand2.removeElementAt(0);
			else
				operand2.setElementAt(felement.substring(2), 0);
			if ( lelement.length() == 2 )
				operand2.removeElementAt(operand2.size()-1);
			else
				operand2.setElementAt(lelement.substring(0, lelement.length()-2), operand2.size()-1);
		}
		else
		{
			if ( operand2.size() == 1 )
				operand2.setElementAt(felement.substring(2, lindex), 0);
			else
			{
				operand2.setElementAt(felement.substring(2), 0);
				operand2.setElementAt(lelement.substring(0, lindex), operand2.size()-1);
			}
		}
	}
	if ( operand2.isEmpty() )
		operand2.addElement("");
				
							  replaceList.addElement(operand1);
							  replaceByList.addElement(operand2);
	         		  operand1 = new Vector();
        			  operand2 = new Vector();
                           }
break;
//#line 2759 "CobolParser.java"
//########## END OF USER-SUPPLIED ACTIONS ##########
    }//switch
    //#### Now let's reduce... ####
    if (yydebug) debug("reduce");
    state_drop(yym);             //we just reduced yylen states
    yystate = state_peek(0);     //get new state
    val_drop(yym);               //corresponding value drop
    yym = yylhs[yyn];            //select next TERMINAL(on lhs)
    if (yystate == 0 && yym == 0)//done? 'rest' state and at first TERMINAL
      {
      debug("After reduction, shifting from state 0 to state "+YYFINAL+"");
      yystate = YYFINAL;         //explicitly say we're done
      state_push(YYFINAL);       //and save it
      val_push(yyval);           //also save the semantic value of parsing
      if (yychar < 0)            //we want another character?
        {
        yychar = yylex();        //get next character
        if (yychar<0) yychar=0;  //clean, if necessary
        if (yydebug)
          yylexdebug(yystate,yychar);
        }
      if (yychar == 0)          //Good exit (if lex returns 0 ;-)
         break;                 //quit the loop--all DONE
      }//if yystate
    else                        //else not done yet
      {                         //get next state and push, for next yydefred[]
      yyn = yygindex[yym];      //find out where to go
      if ((yyn != 0) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn]; //get new state
      else
        yystate = yydgoto[yym]; //else go to new defred
      debug("after reduction, shifting from state "+state_peek(0)+" to state "+yystate+"");
      state_push(yystate);     //going again, so push state & val...
      val_push(yyval);         //for next action
      }
    }//main loop
  return 0;//yyaccept!!
}
//## end of method yyparse() ####################################



//## Constructor ################################################
public CobolParser()
{
}

public CobolParser(boolean debug_me)
{
  yydebug=debug_me;
}
//###############################################################



}
//################### END OF CLASS CobolParser ######################
